#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const chalk_1 = require("chalk");
const compile_1 = require("../lib/compile");
program
    .usage('[project] [option]')
    .option('-t, --type [type]', '静态文件类型: js/css/sass/image/vm/jsLab')
    .option('-e, --env [env]', '设置 NODE_ENV 环境变量 production/testing/development, 首字母也ok')
    .option('--ssr, --ssr [ssr]', '只编译react server端文件')
    .description('编译前端静态文件')
    .action((project, option) => {
    if (!option.type || option.type === true) {
        option.type = 'all';
    }
    switch (program.env) {
        case 'd':
        case 'development':
            process.env.NODE_ENV = 'development';
            break;
        case 't':
        case 'testing':
            process.env.NODE_ENV = 'testing';
            break;
        case 'p':
        case 'production':
            process.env.NODE_ENV = 'production';
            break;
        default:
            process.env.NODE_ENV = 'development';
            break;
    }
    global.__PROJECT__ = project;
    global.__SSR__ = option.ssr ? true : false;
    compile_1.default(option.type);
});
program.on('--help', () => {
    console.log('\n  举个栗子:\n');
    console.log(chalk_1.default.gray('    # 打包本地项目 example 内所有静态文件'));
    console.log('    $ dop compile example  或者简写  dop c example\n');
    console.log(chalk_1.default.gray('    # 打包特定类型静态文件(js/css/images/vm/scss)'));
    console.log('    $ dop compile example --type js  或者  dop c example -t css\n');
});
(function activeHelp() {
    program.parse(process.argv);
    if (program.args.length < 1) {
        return program.help();
    }
})();
