#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const chalk_1 = require("chalk");
const server_1 = require("../lib/server");
program
    .usage('[project] [option]')
    .option('-p, --port [port]', '设置本地服务端口号')
    .option('-s, --ssl', '是否监听本地 443 端口 [开启 https 服务]')
    .option('-e, --env [env]', '设置 NODE_ENV 环境变量 production/testing/development, 首字母也ok')
    .option('--ejs, --ejs [ejs]', '开启react srr模式，默认模板引擎切换为ejs')
    .action((project, option) => {
    switch (program.env) {
        case 'd':
        case 'development':
            process.env.NODE_ENV = 'development';
            break;
        case 't':
        case 'testing':
            process.env.NODE_ENV = 'testing';
            break;
        case 'p':
        case 'production':
            process.env.NODE_ENV = 'production';
            break;
        default:
            process.env.NODE_ENV = 'development';
            break;
    }
    global.__PROJECT__ = project;
    global.__EJS__ = option.ejs ? true : false;
    server_1.default(Number(program.port || 80), program.ssl);
});
program.on('--help', () => {
    console.log('\n  举个栗子:\n');
    console.log(chalk_1.default.gray('    # 启动本地项目 example 服务'));
    console.log('    $ dop server example  或者简写  dop s example\n');
    console.log(chalk_1.default.gray('    # 自定义端口和监听443端口(https)'));
    console.log('    $ dop server example --port 3000       或者  dop server example -p 3000');
    console.log('    $ dop server example --ssl             或者  dop server example -s');
    console.log('    $ dop server example --env production  或者  dop s example -e d\n');
});
(function activeHelp() {
    program.parse(process.argv);
    if (program.args.length < 1) {
        return program.help();
    }
})();
