"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const webpack_sources_1 = require("webpack-sources");
const concatAndOutput = (baseCss, concatCss, output) => {
    const concatStr = new webpack_sources_1.ConcatSource(baseCss, concatCss);
    output && fs.writeFileSync(output, concatStr.source(), 'utf8');
};
class ConcatCss {
    constructor(options = {}) {
        this.options = {};
        this.chunkVersions = {};
        Object.assign(this.options, options);
    }
    apply(compiler) {
        const { entryName = '', sourceFile, baseCss, output } = this.options;
        compiler.plugin("compilation", (compilation) => {
            compilation.plugin("optimize-chunk-assets", (chunks, callback) => {
                if (!sourceFile && !baseCss && !output) {
                    callback();
                    return;
                }
                const changedChunks = chunks.filter((chunk) => {
                    const oldVersion = this.chunkVersions[chunk.name];
                    this.chunkVersions[chunk.name] = chunk.hash;
                    return chunk.hash !== oldVersion;
                });
                const containCssChunk = chunks.filter((chunk) => {
                    return chunk.files.includes(entryName);
                }).map((chunk) => {
                    return chunk.name;
                });
                for (const chunk of changedChunks) {
                    if (containCssChunk.includes(chunk.name)) {
                        if (baseCss) {
                            concatAndOutput(baseCss, compilation.assets[entryName], output);
                            callback();
                            return;
                        }
                        callback();
                        return;
                    }
                }
                callback();
            });
        });
    }
}
exports.default = ConcatCss;
