"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const rimraf = require("rimraf");
const postcss = require("postcss");
const getPostcssPlugins_1 = require("../dependencies/getPostcssPlugins");
const CompileBaseClass_1 = require("../CompileBaseClass");
const readDir_1 = require("../../../util/readDir");
const mkdirs_1 = require("../../../util/mkdirs");
const watch_1 = require("../../../util/watch");
const dophin_core_1 = require("dophin-core");
const { log, debounce } = dophin_core_1.utils;
class CssCompiler extends CompileBaseClass_1.default {
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const that = this;
            const { rootPath, rootConfig, projectConfig = {} } = that.runtime;
            let { entry, type, getString, singleFile } = that.params;
            const project = global.__PROJECT__;
            if (type === undefined) {
                type = entry;
            }
            const outDir = rootConfig && rootConfig.dist.css.replace('{project}', project) || '';
            const entryDir = rootConfig && rootConfig.src.css.replace('{project}', project) || '';
            let entryFiles = [];
            if (!fs.existsSync(entryDir) && !singleFile) {
                return;
            }
            const handleCss = (file) => {
                const postcssPlugins = getPostcssPlugins_1.default({ rootPath, rootConfig, projectConfig }, file);
                const promise = new Promise((resolve, reject) => {
                    log.debug(`开始打包 ${path.basename(file)}`);
                    fs.readFile(file, { encoding: 'utf8' }, (err, css) => {
                        if (err) {
                            reject(err);
                            return;
                        }
                        let outPath = '';
                        if (singleFile) {
                            outPath = singleFile.dest;
                        }
                        else {
                            const _dir = path.relative(entryDir, file);
                            outPath = path.resolve(outDir, _dir);
                        }
                        mkdirs_1.default(path.dirname(outPath));
                        postcss(postcssPlugins).process(css, {
                            from: file,
                            to: outPath,
                        }).then(function (result) {
                            if (!getString) {
                                fs.writeFileSync(outPath, result.css, 'utf8');
                            }
                            resolve(result.css);
                        }).catch((error) => {
                            reject(error);
                        });
                    });
                });
                return promise;
            };
            const asyncCompile = (_entryFiles) => {
                const promises = _entryFiles.map((file) => {
                    return handleCss(file);
                });
                return Promise.all(promises);
            };
            const calEntryFiles = () => {
                entryFiles = [];
                const files = readDir_1.default(entryDir);
                files.forEach((file) => {
                    if (file.name.indexOf('_') !== 0 && file.name.indexOf('.') !== 0) {
                        entryFiles.push(file.path);
                    }
                });
            };
            if (entry) {
                entryFiles.push(entry);
            }
            else {
                calEntryFiles();
            }
            const watchTask = () => {
                watch_1.default({ directories: [entryDir] }, (filePath, mtime) => {
                    if (filePath && path.basename(filePath).indexOf('.') !== 0) {
                        log.info(`${filePath} 有改动, 开始编译并打包到 build 目录`);
                        calEntryFiles();
                        asyncCompile(entryFiles).then(() => {
                            log.success('css打包完成');
                        }).catch((_err) => {
                            log.error(_err);
                        });
                    }
                });
            };
            const compilePromise = () => {
                return new Promise((resolve, reject) => {
                    if (entry) {
                        resolve(asyncCompile(entryFiles));
                    }
                    else {
                        log.debug(`开始清理${outDir}`);
                        rimraf(outDir, (err) => {
                            if (err) {
                                reject(err);
                                return;
                            }
                            resolve(asyncCompile(entryFiles));
                        });
                    }
                });
            };
            switch (type) {
                case 'watch':
                    watchTask();
                    break;
                case 'compile':
                    return compilePromise();
                case 'watchAndCompile':
                    asyncCompile(entryFiles).then(() => {
                        log.success('css打包完成');
                    }).catch((_err) => {
                        log.error(_err);
                    });
                    watchTask();
                default:
                    break;
            }
        });
    }
    afterTaskRun() {
        const { entry = '' } = this.params;
        if (entry.indexOf('.') !== -1) {
            this.result = this.runtime.taskRunning[0];
        }
        else {
            this.result = this.runtime.taskRunning;
        }
    }
}
const wrapper = (entry, type, getString, singleFile) => {
    return new CssCompiler(entry, type, getString, singleFile);
};
exports.default = wrapper;
