"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const webpack = require("webpack");
const extend = require("extend");
const ExtractTextPlugin = require("extract-text-webpack-plugin");
const css_1 = require("../css");
const scss_1 = require("../scss");
const concatCss_1 = require("../compileTool/plugins/concatCss");
const recordModules_1 = require("../compileTool/plugins/recordModules");
const log_1 = require("../../../util/log");
const getWebpackConfig_1 = require("./getWebpackConfig");
let configCache;
const initCompileConfig = ({ rootConfig, rootPath, projectConfig }) => __awaiter(void 0, void 0, void 0, function* () {
    if (configCache) {
        return configCache;
    }
    const rootDir = rootPath + '';
    const webpackDefaultConfig = getWebpackConfig_1.default({ rootConfig, rootPath, projectConfig });
    const webpackPlugins = webpackDefaultConfig.plugins || [];
    const webpackRules = webpackDefaultConfig.module.rules || [];
    if (projectConfig.webpack && projectConfig.webpack.need) {
        delete projectConfig.webpack.need;
    }
    if (projectConfig.react) {
        try {
            if (typeof projectConfig.react.manifest === 'string') {
                projectConfig.react.manifest = [projectConfig.react.manifest];
            }
            projectConfig.react.manifest.forEach((manifest) => {
                const _manfestFile = path.resolve(rootDir, manifest);
                if (fs.existsSync(_manfestFile)) {
                    webpackPlugins.push(new webpack.DllReferencePlugin({
                        context: rootDir,
                        manifest: require(_manfestFile),
                    }));
                }
            });
        }
        catch (error) {
            log_1.default.error(error);
        }
    }
    if (projectConfig.webpack && projectConfig.webpack.loader) {
        Array.isArray(projectConfig.webpack.loader) && projectConfig.webpack.loader.forEach((_loader) => {
            if (_loader.loaderName !== '' && _loader.loaderName !== null && typeof _loader.loaderName !== 'undefined') {
                webpackRules.push({
                    test: _loader.test,
                    use: [
                        {
                            loader: path.resolve(__dirname, "../../../", "lib/compile/compileTool/loader/" + _loader.loaderName + ".js"),
                            options: _loader.options || '',
                        },
                    ],
                    exclude: _loader.exclude || /node_modules/,
                });
            }
            else {
                webpackRules.push(_loader);
            }
        });
        delete projectConfig.webpack.loader;
    }
    if (projectConfig.webpack && projectConfig.webpack.pluginConfig) {
        Array.isArray(projectConfig.webpack.pluginConfig) && projectConfig.webpack.pluginConfig.forEach((_plugin) => {
            if (Array.isArray(_plugin)) {
                const [pluginName, pluginConfig] = _plugin;
                const plugin = eval(pluginName);
                webpackPlugins.push(new plugin(pluginConfig));
            }
            if (typeof _plugin === 'function') {
                webpackPlugins.push(_plugin);
            }
        });
        delete projectConfig.webpack.pluginConfig;
    }
    if (projectConfig.extractCss) {
        const { extractedCssName = '../css/extract.css', sourceFile, output, } = projectConfig.extractCss;
        webpackPlugins.push(new ExtractTextPlugin({
            allChunks: true,
            filename: extractedCssName,
        }));
        let baseCss = '';
        if (/\.scss/.test(sourceFile)) {
            baseCss = yield scss_1.default(sourceFile, 'compile', true).run();
        }
        else if (/\.css/.test(sourceFile)) {
            baseCss = yield css_1.default(sourceFile, 'compile', true).run();
        }
        sourceFile && output && webpackPlugins.push(new concatCss_1.default({
            baseCss,
            entryName: extractedCssName,
            output,
        }));
    }
    webpackPlugins.push(new recordModules_1.default({ rootDir }));
    const config = extend(true, {
        webpack: extend(true, webpackDefaultConfig, projectConfig.webpack),
    }, projectConfig);
    configCache = config;
    return config;
});
exports.default = initCompileConfig;
