"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const postImport = require("postcss-import");
const stylelint = require("stylelint");
const cssnext = require("postcss-cssnext");
const cssmixins = require("postcss-mixins");
const cssnano = require("cssnano");
const px2rem = require("postcss-pxtorem");
const postcssFlexbugsFixes = require("postcss-flexbugs-fixes");
const dophin_core_1 = require("dophin-core");
let pluginCache;
const getPlugins = ({ rootConfig, rootPath, projectConfig }, filePath) => {
    if (pluginCache) {
        return pluginCache;
    }
    const cssConfig = projectConfig.sass || projectConfig.css;
    const px2remConfig = {
        rootValue: (cssConfig.rem.rootValue) || ((16 * cssConfig.rem.psdWidth) / 375) || 32,
        unitPrecision: 4,
        propList: ['*'],
        selectorBlackList: [],
        replace: true,
        mediaQuery: false,
        minPixelValue: 0,
    };
    const dophinRootPath = path.resolve(__dirname, "../../../");
    const plugins = [
        postImport,
        cssmixins,
        stylelint({
            fix: true,
            configFile: path.resolve(rootPath, '.stylelintrc'),
            configBasedir: dophinRootPath,
            codeFilename: path.basename(filePath || ''),
        }),
        postcssFlexbugsFixes,
        cssnext({
            browsers: (rootConfig && rootConfig.browsers) || dophin_core_1.constants.browsers,
            warnForDuplicates: false,
        }),
    ];
    if (cssConfig.rem && cssConfig.rem.need) {
        plugins.push(px2rem(px2remConfig));
    }
    if (process.env.NODE_ENV === 'production') {
        plugins.push(cssnano({
            preset: [
                "default",
                { cssDeclarationSorter: false },
            ],
        }));
    }
    pluginCache = plugins;
    return plugins;
};
exports.default = getPlugins;
