"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const webpack = require("webpack");
const ExtractTextPlugin = require("extract-text-webpack-plugin");
const getBabelConfig_1 = require("./getBabelConfig");
const log_1 = require("../../../util/log");
const getPostcssPlugins_1 = require("./getPostcssPlugins");
const getConfig = ({ rootConfig, rootPath, projectConfig }) => {
    const rootDir = rootPath + '';
    const dophinRootPath = path.resolve(__dirname, "../../../");
    const { extractCss } = projectConfig;
    const _alias = {}, commonDir = [
        path.resolve(rootDir, 'project/common/components'),
        path.resolve(rootDir, 'project/common/plugin'),
    ];
    commonDir.forEach((_dir) => {
        if (!fs.existsSync(_dir)) {
            return;
        }
        const files = fs.readdirSync(_dir);
        log_1.default.debug(`读取 project/commmon/${path.basename(_dir)} 内 webpack alias`);
        files.forEach((file) => {
            if (file.indexOf('.') !== -1) {
                file.split('.')[0] && (_alias[file.split('.')[0]] = path.resolve(_dir, file.split('.')[0]));
            }
            else {
                _alias[file] = path.resolve(_dir, file);
            }
        });
    });
    const childrenProjectsDir = path.resolve(rootPath, 'project');
    if (fs.existsSync(childrenProjectsDir)) {
        const childrenProjects = fs.readdirSync(childrenProjectsDir);
        childrenProjects.forEach((project) => {
            if (fs.existsSync(path.resolve(childrenProjectsDir, project, 'config.js'))) {
                _alias[`@${project}`] = path.resolve(childrenProjectsDir, project, 'source', 'js');
            }
        });
    }
    const webpackPlugins = [];
    let webpackDevtool = false;
    webpackPlugins.push(new webpack.DefinePlugin({
        'process.env': {
            NODE_ENV: JSON.stringify(process.env.NODE_ENV),
        },
    }));
    if (process.env.NODE_ENV === 'production') {
        webpackDevtool = false;
        webpackPlugins.push(new webpack.optimize.UglifyJsPlugin());
    }
    else if (process.env.NODE_ENV === 'development') {
        webpackDevtool = 'cheap-module-eval-source-map';
    }
    else if (process.env.NODE_ENV === 'testing') {
        webpackDevtool = 'cheap-module-eval-source-map';
    }
    const babelLoaderOptions = Object.assign(Object.assign({}, getBabelConfig_1.getBabelConfig(rootConfig)), { cacheDirectory: true });
    const cssLoaders = [
        {
            loader: 'css-loader',
            options: { importLoaders: 1 },
        },
        {
            loader: 'postcss-loader',
            options: {
                ident: 'postcss',
                plugins: getPostcssPlugins_1.default({ rootConfig, rootPath, projectConfig }),
            },
        },
    ];
    let tsconfigPath = path.resolve(rootDir, `project/${global.__PROJECT__}/tsconfig.json`);
    if (!fs.existsSync(tsconfigPath)) {
        tsconfigPath = 'tsconfig.json';
    }
    const config = {
        context: dophinRootPath,
        module: {
            rules: [
                {
                    test: /\.bundle\.js?$/,
                    use: [
                        {
                            loader: 'bundle-loader?lazy',
                            options: {
                                lazy: true,
                                name: '[name]',
                            },
                        },
                    ],
                    exclude: /node_modules/,
                },
                {
                    test: /\.js$/,
                    loader: 'babel-loader',
                    exclude: /node_modules/,
                    options: babelLoaderOptions,
                },
                {
                    test: /\.art$/,
                    use: [
                        {
                            loader: 'babel-loader',
                            options: babelLoaderOptions,
                        },
                        {
                            loader: 'art-template-loader',
                            options: {
                                loader: require('art-template-loader'),
                                imports: path.resolve(rootDir, 'node_modules', 'art-template/lib/runtime'),
                            },
                        },
                    ],
                },
                {
                    test: /\.jsx$/,
                    loader: 'babel-loader',
                    exclude: /node_modules/,
                    options: babelLoaderOptions,
                },
                {
                    test: /\.bundle\.tsx?$/,
                    use: [{
                            loader: 'bundle-loader?lazy',
                            options: {
                                lazy: true,
                                name: '[name]',
                            },
                        }],
                    exclude: /node_modules/,
                },
                {
                    test: /\.tsx?$/,
                    loader: 'awesome-typescript-loader',
                    exclude: /node_modules/,
                    options: {
                        configFileName: tsconfigPath,
                    },
                },
                {
                    test: /\.css$/,
                    use: (extractCss ?
                        ExtractTextPlugin.extract({
                            fallback: 'style-loader',
                            use: 'css-loader',
                        }) :
                        ['style-loader', ...cssLoaders]),
                },
                {
                    test: /\.scss$/,
                    use: (extractCss ?
                        ExtractTextPlugin.extract({
                            fallback: 'style-loader',
                            use: [...cssLoaders, 'sass-loader'],
                        }) :
                        ['style-loader', ...cssLoaders, 'sass-loader']),
                }, {
                    test: /\.(png|jpg|gif)$/,
                    use: [{
                            loader: 'url-loader',
                            options: {
                                limit: 8192,
                            },
                        }],
                },
            ],
        },
        devtool: webpackDevtool,
        plugins: webpackPlugins,
        resolve: {
            alias: _alias,
            extensions: ['.js', '.ts', '.tsx', '.jsx', '.css', '.scss'],
        },
    };
    return config;
};
exports.default = getConfig;
