"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const scss_1 = require("./scss");
const js_1 = require("./js");
const compileLab_1 = require("./js/compileLab");
const css_1 = require("./css");
const vm_1 = require("./vm");
const images_1 = require("./images");
const dophin_core_1 = require("dophin-core");
const { log } = dophin_core_1.utils;
const run = (type = '') => {
    switch (type.toLocaleLowerCase()) {
        case 'watchall':
            js_1.default('', 'watch').run();
            compileLab_1.default('watch').run();
            css_1.default('', 'watch').run();
            scss_1.default('', 'watch').run();
            vm_1.default('watch').run();
            images_1.default('watch').run();
            break;
        case 'all':
            Promise.all([
                js_1.default('', 'compile').run(),
                compileLab_1.default('compile').run(),
                css_1.default('', 'compile').run(),
                scss_1.default('', 'compile').run(),
                vm_1.default('compile').run(),
                images_1.default('compile').run(),
            ]).then(() => {
                log.success('静态文件全部打包完成');
            }).catch((err) => {
                log.error('打包错误\n');
                log.error(err.stack);
            });
            break;
        case 'js':
            js_1.default('', 'compile').run().then(() => {
                log.success('js打包完成');
            }).catch((err) => {
                log.error(err.stack);
                log.error('js打包错误');
            });
            break;
        case 'jslab':
            compileLab_1.default('compile').run().then(() => {
                log.success('js-lab打包完成');
            }).catch((err) => {
                log.error(err.stack);
                log.error('js-lab打包错误');
            });
            break;
        case 'css':
            css_1.default('', 'compile').run().then(() => {
                log.success('css打包完成');
            }).catch((err) => {
                log.error(err.stack);
            });
            break;
        case 'sass':
        case 'scss':
            scss_1.default('', 'compile').run().then(() => {
                log.success('scss打包完成');
            });
            break;
        case 'vm':
            vm_1.default('compile').run().then(() => {
                log.success('vm打包完成');
            }).catch((err) => {
                log.error(err);
            });
            break;
        case 'images':
        case 'image':
        case 'img':
        case 'png':
        case 'jpg':
            images_1.default('compile').run().then(() => {
                log.success('images打包完成');
            }).catch((err) => {
                log.error(err);
            });
        default:
            break;
    }
};
exports.default = run;
