"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const babel_core_1 = require("babel-core");
const uglify_js_1 = require("uglify-js");
const CompileBaseClass_1 = require("../CompileBaseClass");
const getBabelConfig_1 = require("../dependencies/getBabelConfig");
const readDir_1 = require("../../../util/readDir");
const mkdirs_1 = require("../../../util/mkdirs");
const watch_1 = require("../../../util/watch");
const dophin_core_1 = require("dophin-core");
const { log } = dophin_core_1.utils;
class JsLabCompiler extends CompileBaseClass_1.default {
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rootPath, rootConfig, projectConfig = {} } = this.runtime;
            let { entry, type } = this.params;
            const project = global.__PROJECT__;
            if (type === undefined) {
                type = entry;
            }
            const outDir = rootConfig && rootConfig.dist.js.replace('{project}', project) || '';
            const entryDir = rootConfig && rootConfig.src.jsLab.replace('{project}', project) || '';
            if (!fs.existsSync(entryDir)) {
                return;
            }
            const handleJs = (filepath) => {
                const fileBasename = path.basename(filepath);
                const promise = new Promise((resolve, reject) => {
                    if (filepath && fileBasename.indexOf('.') !== 0) {
                        const src = filepath, _dir = path.relative(entryDir, src), dest = path.resolve(outDir, _dir);
                        mkdirs_1.default(path.dirname(dest));
                        if (fs.existsSync(src)) {
                            log.debug(`开始打包${fileBasename}`);
                            fs.readFile(src, { encoding: 'utf8' }, (err, codes) => {
                                if (err) {
                                    reject(err);
                                    return;
                                }
                                let _codes = codes;
                                if (Array.isArray(projectConfig.jsLabBabel) && projectConfig.jsLabBabel.includes(path.basename(src))) {
                                    try {
                                        _codes = babel_core_1.transform(_codes, getBabelConfig_1.getBabelConfig()).code + '';
                                    }
                                    catch (error) {
                                        log.error(error.message);
                                    }
                                }
                                if (process.env.NODE_ENV === 'production') {
                                    try {
                                        _codes = uglify_js_1.minify(_codes).code;
                                    }
                                    catch (error) {
                                        log.error(error.message);
                                    }
                                }
                                fs.writeFileSync(dest, _codes, 'utf8');
                                resolve(fileBasename);
                            });
                        }
                        else {
                            reject(new Error(`${fileBasename}不存在`));
                        }
                    }
                });
                return promise;
            };
            const watchTask = () => {
                watch_1.default({ directories: [entryDir] }, (filePath, mtime) => {
                    if (!fs.existsSync(filePath)) {
                        log.warn(`${path.basename(filePath)}已删除, build目录内历史打包文件仍然保留`);
                        return;
                    }
                    handleJs(filePath).catch((err) => {
                        log.error(err);
                    });
                });
            };
            const asyncCompile = () => {
                const files = readDir_1.default(entryDir);
                const promises = files.map((file) => {
                    return handleJs(file.path);
                });
                return Promise.all(promises);
            };
            switch (type) {
                case 'watch':
                    watchTask();
                    break;
                case 'compile':
                    return asyncCompile();
                case 'watchAndCompile':
                    asyncCompile().then(() => {
                        log.success('js-lab打包完成');
                    }).catch((err) => {
                        log.error('js-lab打包错误');
                    });
                    watchTask();
                    break;
                default:
                    return asyncCompile();
            }
        });
    }
}
const wrapper = (entry, type) => {
    return new JsLabCompiler(entry, type);
};
exports.default = wrapper;
