"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const webpack = require("webpack");
const extend = require("extend");
const CompileBaseClass_1 = require("../CompileBaseClass");
const getCompileConfig_1 = require("../dependencies/getCompileConfig");
const readDir_1 = require("../../../util/readDir");
const getJsEntryName_1 = require("../../../util/getJsEntryName");
const watch_1 = require("../../../util/watch");
const dophin_core_1 = require("dophin-core");
const { log } = dophin_core_1.utils;
const concat_1 = require("./tools/concat");
const webpackLog = (err, stats) => {
    if (err) {
        log.error(err.message);
        return;
    }
    console.log('webpack打包结果\n' + stats.toString({
        hash: false,
        chunks: false,
        colors: true,
        children: false,
        modules: false,
    }));
};
class JsCompiler extends CompileBaseClass_1.default {
    beforeTaskRun() {
        return __awaiter(this, void 0, void 0, function* () {
            const { rootPath, rootConfig, projectConfig = {} } = this.runtime;
            const { entry, type } = this.params;
            const project = global.__PROJECT__;
            const entryDir = rootConfig && rootConfig.src.js.replace('{project}', project) || '';
            const outDir = rootConfig && rootConfig.dist.js.replace('{project}', project);
            const webpackEntry = {};
            const keepOriginalFilename = projectConfig.keepOriginalFilename;
            if (entry) {
                webpackEntry[getJsEntryName_1.default(entry, keepOriginalFilename)] = entry;
            }
            else {
                if (!fs.existsSync(entryDir)) {
                    return;
                }
                const files = readDir_1.default(entryDir);
                files.forEach((file) => {
                    if (file.name.indexOf('entry') === 0) {
                        const relPath = path.relative(entryDir, path.dirname(file.path));
                        webpackEntry[(relPath ? `${relPath}/` : '') + getJsEntryName_1.default(file.name, keepOriginalFilename)] = file.path;
                    }
                });
            }
            const outputFilename = '[name].js';
            let outputChunkFilename = '[name].js';
            if (projectConfig.version && process.env.NODE_ENV === 'production') {
                outputChunkFilename = '[name].[chunkhash:5].js';
            }
            const calculatConfig = {
                entry: webpackEntry,
                output: {
                    path: outDir,
                    filename: outputFilename,
                    publicPath: `/${project}/js/`,
                    chunkFilename: outputChunkFilename,
                },
            };
            let ssrConfig = {};
            console.log(global.__SSR__);
            if (global.__SSR__ === true) {
                for (const key in webpackEntry) {
                    if (key.toLowerCase().indexOf('ssr') <= -1) {
                        delete webpackEntry[key];
                    }
                }
                ssrConfig = {
                    output: {
                        libraryTarget: 'commonjs2',
                    },
                    target: 'node',
                };
            }
            else {
                for (const key in webpackEntry) {
                    if (key.toLowerCase().indexOf('ssr') > -1) {
                        delete webpackEntry[key];
                    }
                }
            }
            const compileConfig = yield getCompileConfig_1.default({ rootPath, rootConfig, projectConfig });
            const webpackConfig = extend(true, calculatConfig, compileConfig.webpack, ssrConfig);
            this.webpackConfig = webpackConfig;
            if (projectConfig.bundle) {
                const bundles = projectConfig.bundle;
                const bundleDirInBuild = 'bundle';
                const bundleConfigFilepath = path.resolve(entryDir, 'bundle_config.js');
                const bundleEntry = {};
                const bundleConfig = {};
                for (const _b of bundles) {
                    const _bundleRawName = getJsEntryName_1.default(path.basename(_b.entry));
                    let _bundleFullName = _bundleRawName;
                    if (_b.version) {
                        _bundleFullName += `_${String(_b.version).replace(/\./g, '_')}`;
                    }
                    bundleEntry[_bundleFullName] = _b.entry;
                    bundleConfig[_bundleRawName] = `${bundleDirInBuild}/${_bundleFullName}.js`;
                }
                const bundleWebpackConfig = extend(false, {}, webpackConfig, {
                    entry: bundleEntry, output: {
                        path: path.join(outDir || '', bundleDirInBuild),
                    },
                });
                const compiler = webpack(bundleWebpackConfig);
                const compilePromise = () => {
                    return new Promise((resolve, reject) => {
                        compiler.run((_err, stats) => {
                            webpackLog(_err, stats);
                            if (_err) {
                                reject(_err);
                                return;
                            }
                            resolve();
                        });
                    });
                };
                switch (type) {
                    case 'watch':
                    case 'watchAndCompile':
                        compiler.watch({}, (err, stats) => {
                            webpackLog(err, stats);
                        });
                        break;
                    default:
                        yield compilePromise();
                }
                const bundleConfigContent = `/**
 * bundle配置
 * @version ${new Date()}
 */

module.export = ${JSON.stringify(bundleConfig)};
`;
                fs.writeFileSync(bundleConfigFilepath, bundleConfigContent, { encoding: 'utf8' });
            }
        });
    }
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const { projectConfig = {} } = this.runtime;
            const { entry, type } = this.params;
            const concatConfig = projectConfig.jsConcat;
            const isConcatFiles = concatConfig ? true : false;
            const compiler = webpack(this.webpackConfig);
            const compilePromise = () => {
                return new Promise((resolve, reject) => {
                    compiler.run((_err, stats) => {
                        webpackLog(_err, stats);
                        if (_err) {
                            reject(_err);
                            return;
                        }
                        resolve();
                    });
                });
            };
            const concatPromise = () => {
                const promises = [];
                Array.isArray(concatConfig) && concatConfig.forEach((task) => {
                    promises.push(concat_1.default(task.target, task.source));
                });
                return Promise.all(promises);
            };
            const watchAndConcat = () => {
                const watchTask = {};
                Array.isArray(concatConfig) && concatConfig.forEach((task) => {
                    task.source.forEach((sourceFile) => {
                        if (watchTask[sourceFile]) {
                            watchTask[sourceFile].push(task);
                        }
                        else {
                            watchTask[sourceFile] = [task];
                        }
                    });
                });
                for (const sourceFile in watchTask) {
                    watch_1.default({
                        files: [sourceFile],
                    }, (filePath, mtime) => {
                        watchTask[sourceFile].forEach((task) => {
                            concat_1.default(task.target, task.source);
                        });
                    });
                }
            };
            const hasEntryFile = !!entry || Object.keys(this.webpackConfig.entry || {}).length > 0;
            switch (type) {
                case 'watch':
                case 'watchAndCompile':
                    isConcatFiles && watchAndConcat();
                    hasEntryFile && compiler.watch({}, (err, stats) => {
                        webpackLog(err, stats);
                    });
                    break;
                default:
                    const compilePromises = [];
                    isConcatFiles && compilePromises.push(concatPromise());
                    hasEntryFile && compilePromises.push(compilePromise());
                    return Promise.all(compilePromises);
            }
        });
    }
}
const wrapper = (entry, type) => {
    return new JsCompiler(entry, type);
};
exports.default = wrapper;
