"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const rimraf = require("rimraf");
const postcss = require("postcss");
const compass_1 = require("./tools/compass");
const sass_1 = require("./tools/sass");
const css_1 = require("../css");
const CompileBaseClass_1 = require("../CompileBaseClass");
const getPostcssPlugins_1 = require("../dependencies/getPostcssPlugins");
const mkdirs_1 = require("../../../util/mkdirs");
const readDir_1 = require("../../../util/readDir");
const dophin_core_1 = require("dophin-core");
const { log, debounce } = dophin_core_1.utils;
class SassCompiler extends CompileBaseClass_1.default {
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rootPath, rootConfig, projectConfig = {} } = this.runtime;
            const { entry, type, getString } = this.params;
            const project = global.__PROJECT__;
            const outDir = rootConfig && rootConfig.dist.css.replace('{project}', project) || '';
            const entryDir = rootConfig && rootConfig.src.scss.replace('{project}', project) || '';
            if (!fs.existsSync(entryDir)) {
                return;
            }
            const sassCompileSingleFile = (file) => __awaiter(this, void 0, void 0, function* () {
                const src = file, _dir = path.relative(entryDir, src);
                let dest = path.resolve(outDir, _dir);
                dest = dest.replace(path.extname(dest), '.css');
                mkdirs_1.default(path.dirname(dest));
                const postcssPlugins = getPostcssPlugins_1.default({ rootPath, rootConfig, projectConfig }, file);
                log.info(`sass: 使用node-sass打包 ${path.basename(file)} 中...`);
                const css = yield sass_1.default(file);
                log.info(`sass: 使用postcss处理 ${path.basename(file)} `);
                const usePostcss = new Promise((resolve, reject) => {
                    postcss(postcssPlugins).process(css, {
                        from: file,
                        to: dest,
                    }).then(function (result) {
                        if (!getString) {
                            fs.writeFileSync(dest, result.css, 'utf8');
                        }
                        resolve(result.css);
                    }).catch((error) => {
                        reject(error);
                    });
                });
                return yield usePostcss;
            });
            const asyncCompile = (filepath) => __awaiter(this, void 0, void 0, function* () {
                if (projectConfig.sass && projectConfig.sass && projectConfig.sass.compass && projectConfig.sass.compass.need) {
                    log.info('sass: compass打包中...');
                    yield compass_1.default({
                        css: outDir,
                        sass: entryDir,
                        project: rootPath || process.cwd(),
                    });
                    log.info('sass: 使用postcss处理compass打包后文件们');
                    const compassPromises = readDir_1.default(outDir).map(((file) => __awaiter(this, void 0, void 0, function* () {
                        return css_1.default(file.path, 'compile', false, { src: file.path, dest: file.path }).run();
                    })));
                    return Promise.all(compassPromises);
                }
                const sassPromises = readDir_1.default(entryDir).map(((file) => __awaiter(this, void 0, void 0, function* () {
                    if (path.basename(file.path).indexOf('_') === 0) {
                        return;
                    }
                    return yield sassCompileSingleFile(file.path);
                })));
                return Promise.all(sassPromises);
            });
            const watchTask = () => {
                if (!projectConfig.sass) {
                    return;
                }
                fs.watch(entryDir, {
                    encoding: 'utf8',
                    recursive: true,
                    persistent: true,
                }, debounce((eventType) => __awaiter(this, void 0, void 0, function* () {
                    yield asyncCompile();
                }), 100));
            };
            const compileTask = () => {
                return new Promise((resolve, reject) => {
                    if (entry) {
                        resolve(sassCompileSingleFile(entry));
                        return;
                    }
                    if (!projectConfig.sass) {
                        resolve('');
                    }
                    else {
                        log.debug(`开始清理${outDir}`);
                        rimraf(outDir, (err) => {
                            if (err) {
                                log.error(err.message);
                                reject(err);
                                return;
                            }
                            resolve(asyncCompile());
                        });
                    }
                });
            };
            switch (type) {
                case 'watch':
                    watchTask();
                    break;
                case 'compile':
                    return compileTask();
                case 'watchAndCompile':
                    compileTask().then();
                    watchTask();
                    break;
                default:
                    return compileTask();
            }
        });
    }
    afterTaskRun() {
        const { entry = '' } = this.params;
        this.result = this.runtime.taskRunning;
    }
}
const wrapper = (entry, type, getString) => {
    return new SassCompiler(entry, type, getString);
};
exports.default = wrapper;
