"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const child_process_1 = require("child_process");
const which_1 = require("which");
const log_1 = require("../../../../util/log");
const compass = (options) => __awaiter(void 0, void 0, void 0, function* () {
    const defaults = {
        style: false,
        comments: false,
        relative: true,
        css: 'css',
        sass: 'sass',
        image: false,
        generated_images_path: false,
        http_path: false,
        javascript: false,
        font: false,
        import_path: false,
        config_file: false,
        require: false,
        logging: true,
        load_all: false,
        project: process.cwd(),
        bundle_exec: false,
        debug: false,
        time: false,
        sourcemap: false,
        boring: false,
        force: false,
        task: 'compile',
    };
    options = Object.assign(defaults, options);
    let compassCommand = 'compass';
    try {
        compassCommand = which_1.sync(compassCommand);
    }
    catch (err) {
        return new Error('请安装sass和compass');
    }
    const optionsArr = [];
    optionsArr.push('compile');
    if (process.platform === 'win32') {
        optionsArr.push(options.project.replace(/\\/g, '/'));
    }
    else {
        optionsArr.push(options.project);
    }
    if (options.debug) {
        optionsArr.push('--debug-info');
    }
    if (options.time) {
        optionsArr.push('--time');
    }
    if (options.boring) {
        optionsArr.push('--boring');
    }
    if (options.sourcemap) {
        optionsArr.push('--sourcemap');
    }
    if (options.style) {
        optionsArr.push('--output-style', options.style);
    }
    optionsArr.push('--css-dir', path.normalize(options.css));
    optionsArr.push('--sass-dir', path.normalize(options.sass));
    const promise = new Promise((resolve, reject) => {
        const child = child_process_1.spawn(compassCommand, optionsArr, { cwd: options.project || process.cwd() });
        let stdout = '', stderr = '';
        if (options.logging) {
            child.stdout.setEncoding('utf8');
            child.stdout.on('data', (data) => {
                stdout += data;
                log_1.default.debug(data.toString());
            });
            child.stderr.setEncoding('utf8');
            child.stderr.on('data', (data) => {
                stderr += data;
                if (!data.toString().match(/^\u001b\[\d+m$/)) {
                    log_1.default.error(data.toString());
                }
            });
        }
        child.on('close', (code) => {
            resolve(stdout);
        });
    });
    return yield promise;
});
exports.default = compass;
