"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const rimraf = require("rimraf");
const CompileBaseClass_1 = require("../CompileBaseClass");
const readDir_1 = require("../../../util/readDir");
const mkdirs_1 = require("../../../util/mkdirs");
const watch_1 = require("../../../util/watch");
const dophin_core_1 = require("dophin-core");
const { log } = dophin_core_1.utils;
class VelocityCompiler extends CompileBaseClass_1.default {
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rootPath, rootConfig, projectConfig = {} } = this.runtime;
            let { entry, type } = this.params;
            const project = global.__PROJECT__;
            if (type === undefined) {
                type = entry;
            }
            const outDir = rootConfig && rootConfig.dist.vm.replace('{project}', project) || '';
            const entryDir = rootConfig && rootConfig.src.vm.replace('{project}', project) || '';
            if (!fs.existsSync(entryDir)) {
                return;
            }
            const handleVM = (filename) => {
                const promise = new Promise((resolve, reject) => {
                    const fileBasename = path.basename(filename);
                    if (filename && fileBasename.indexOf('.') !== 0) {
                        const src = filename, _dir = path.relative(entryDir, src), dest = path.resolve(outDir, _dir);
                        mkdirs_1.default(path.dirname(dest));
                        if (fs.existsSync(src)) {
                            log.debug(`开始打包${fileBasename}`);
                            const readStream = fs.createReadStream(src), writeStream = fs.createWriteStream(dest);
                            readStream.pipe(writeStream);
                            readStream.on('error', function (e) {
                                reject(e);
                            });
                            readStream.on('end', function (e) {
                                resolve(fileBasename);
                            });
                        }
                        else {
                            reject(new Error(`${fileBasename}不存在`));
                        }
                    }
                });
                return promise;
            };
            const watchTask = () => {
                watch_1.default({ directories: [entryDir] }, (filePath, mtime) => {
                    handleVM(filePath).catch((err) => {
                        log.error(err);
                    });
                });
            };
            const asyncCompile = () => {
                const files = readDir_1.default(entryDir);
                const promises = files.map((file) => {
                    return handleVM(file.path);
                });
                return Promise.all(promises);
            };
            const compilePromise = () => {
                return new Promise((resolve, reject) => {
                    log.debug(`开始清理${outDir}`);
                    rimraf(outDir, (err) => {
                        if (err) {
                            reject(err);
                            return;
                        }
                        resolve(asyncCompile());
                    });
                });
            };
            switch (type) {
                case 'watch':
                    watchTask();
                    break;
                case 'compile':
                    return compilePromise();
                case 'watchAndCompile':
                    asyncCompile().then(() => {
                        log.success('vm打包完成');
                    }).catch((_err) => {
                        log.error(_err);
                    });
                    watchTask();
                    break;
                default:
                    return compilePromise();
            }
        });
    }
}
const wrapper = (entry, type) => {
    return new VelocityCompiler(entry, type);
};
exports.default = wrapper;
