"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dophin_core_1 = require("dophin-core");
const GetGitStatus_1 = require("./dependencies/GetGitStatus");
const GetDeployConfig_1 = require("./dependencies/GetDeployConfig");
const log_1 = require("../../util/log");
class DeployClass extends dophin_core_1.DophinBaseClass {
    constructor(taskKey, common) {
        super({ taskKey, common });
    }
    dophinInited() {
        const { allProjects, rootPath, rootConfig } = this.runtime;
        const { taskKey, common } = this.params;
        if (!rootConfig) {
            log_1.default.error('获取项目根目录do.config.js配置文件出错');
            return false;
        }
        log_1.default.info('开始初始化deploy配置信息');
        this.initDeployConfig(taskKey);
        if (!this.deployConfig) {
            log_1.default.error(`deploy.${taskKey} 配置错误, 请检查`);
            return false;
        }
        const project = global.__PROJECT__;
        if (!project) {
            log_1.default.error('DeployClass 缺少参数 project');
            return false;
        }
        if (Array.isArray(allProjects) && !allProjects.includes(project)) {
            log_1.default.error(`${rootPath} 下找不到子项目 ${project}`);
        }
        log_1.default.info('开始初始化开发环境git信息');
        this.initGit();
    }
    initGit() {
        this.git = GetGitStatus_1.default();
    }
    initDeployConfig(taskKey) {
        const { rootConfig, rootPath = '' } = this.runtime;
        const deployConfig = GetDeployConfig_1.default(taskKey, rootPath, rootConfig);
        if (deployConfig) {
            this.deployConfig = deployConfig;
        }
        else {
            log_1.default.error('初始化deploy配置信息失败');
        }
    }
}
exports.default = DeployClass;
