"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const log_1 = require("../../../util/log");
let configCache;
const initConfig = (taskKey, rootPath, rootConfig) => {
    if (configCache) {
        return configCache;
    }
    const config = rootConfig && rootConfig.deploy[taskKey];
    const rootDir = rootPath + '';
    if (!config) {
        log_1.default.error(`检测不到 deploy.${taskKey} 的配置(do.config.js内添加)`);
        return null;
    }
    const project = config.projectName || taskKey;
    global.__PROJECT__ = project;
    config.projNameInStaticDirMap = config.projNameInStaticDirMap || {};
    const [projectNameInBackStatic = project, projectNameInFrontStatic = project] = [config.projNameInStaticDirMap.back, config.projNameInStaticDirMap.front];
    config.javaWebName = config['javaWebName'] || `${project}-web`;
    config.source = path.resolve(rootDir, `project/${project}/source`);
    config.build = path.resolve(rootDir, `project/${project}/build`);
    if (config.CONFIG.git.includes('back')) {
        if (!config.backGitDir) {
            log_1.default.error(`您没有配置 backGitDir 参数`);
            return null;
        }
        const backGitDir = path.resolve(rootDir, config.backGitDir);
        if (!fs.existsSync(backGitDir)) {
            log_1.default.error(`backGitDir: 不存在目录: ${backGitDir}`);
            return null;
        }
        config.backGitDir = backGitDir;
        config.backStaticDir = path.resolve(config.backGitDir, `${config.javaWebName}/src/main/webapp`, config['backStaticDirHasProjName'] === false ? '' : projectNameInBackStatic);
        config.backVMdir = path.resolve(config.backGitDir, `${config.javaWebName}/src/main/webapp`, config['vmDir'] || 'WEB-INF/vm');
        config.backVMLayoutDir = path.resolve(config.backGitDir, `${config.javaWebName}/src/main/webapp`, config['vmLayoutDir'] || 'WEB-INF/vm/layout');
        config.backVMTplDir = path.resolve(config.backGitDir, `${config.javaWebName}/src/main/webapp`, config['vmTplDir'] || `WEB-INF/vm/m/${project}`);
    }
    if (config.CONFIG.git.includes('front')) {
        if (!config.frontGitDir) {
            log_1.default.error(`您没有配置 frontGitDir 参数`);
        }
        const frontGitDir = path.resolve(rootDir, config.frontGitDir);
        if (!fs.existsSync(frontGitDir)) {
            log_1.default.error(`frontGitDir: 不存在目录: ${frontGitDir}`);
            return null;
        }
        config.frontGitDir = frontGitDir;
        config.frontStaticDir = path.resolve(config.frontGitDir, 'src/main/webapp', projectNameInFrontStatic);
    }
    configCache = config;
    return config;
};
exports.default = initConfig;
