"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const child_process_1 = require("child_process");
const log_1 = require("../../../util/log");
let gitCache;
const getGitStatus = () => {
    if (gitCache) {
        return gitCache;
    }
    const _git = {
        version: '',
        dirty: 0,
        history: [],
        modified: [],
    };
    const commitIdLog = child_process_1.execSync('git rev-parse --short HEAD', {
        encoding: 'utf8',
    });
    const statusLog = child_process_1.execSync('git status -s', {
        encoding: 'utf8',
    });
    const commitHistoryLog = child_process_1.execSync('git log --pretty=format:"%h -#- %an -#- %ai -#- %s" -100', {
        encoding: 'utf8',
    });
    const EOL = os.EOL;
    statusLog.split(EOL).filter((data) => {
        return !!data;
    }).forEach((logs) => {
        logs = logs.replace(/^\s?/, '');
        const _status = logs.split(/\s{1,2}/);
        if (!_git[_status[0]]) {
            _git[_status[0]] = [];
        }
        _git[_status[0]].push(_status[1]);
        _git['modified'].push(_status[1]);
    });
    for (const type in _git) {
        if (type !== 'version' && type !== 'dirty') {
            _git.dirty += _git[type].length;
        }
    }
    commitHistoryLog.split(EOL).filter((data) => {
        return !!data;
    }).forEach((logs) => {
        const historyInfoArray = logs.split(/\s-#-\s/);
        _git.history.push(historyInfoArray);
    });
    _git.version = commitIdLog.replace('\n', '');
    log_1.default.info(`当前 git commit: ${_git.version}`);
    _git.dirty && log_1.default.warn(`当前 git 还有 ${_git.dirty} 个文件没有提交, 请尽快commit并push, 否则您发布的代码可能被别人覆盖`);
    gitCache = _git;
    return _git;
};
exports.default = getGitStatus;
