"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const DeployBaseClass_1 = require("./DeployBaseClass");
const preCompile_1 = require("./preCompile");
const compile_1 = require("./compile");
const postCompile_1 = require("./postCompile");
const postDeploy_1 = require("./postDeploy");
const log_1 = require("../../util/log");
class Deploy extends DeployBaseClass_1.default {
    taskRunning(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rootPath, rootConfig, projectConfig = {} } = this.runtime;
            try {
                yield preCompile_1.default(rootConfig, this.deployConfig);
                log_1.default.success('静态文件预处理结束');
            }
            catch (error) {
                log_1.default.error(error.stack);
            }
            try {
                yield compile_1.default(projectConfig, this.deployConfig);
                log_1.default.success('静态文件打包结束');
            }
            catch (error) {
                log_1.default.error(error.stack);
            }
            try {
                const postDeployPromises = [];
                if (this.deployConfig.CONFIG.git.includes('back')) {
                    const backendPromise = () => __awaiter(this, void 0, void 0, function* () {
                        const deployResult = (yield postCompile_1.default(this.deployConfig, 'back')) || null;
                        log_1.default.success('[后端]静态文件部署结束');
                        return yield postDeploy_1.default(projectConfig, this.deployConfig, deployResult, this.git, 'back');
                    });
                    postDeployPromises.push(backendPromise());
                }
                if (this.deployConfig.CONFIG.git.includes('front')) {
                    const frontendPromise = () => __awaiter(this, void 0, void 0, function* () {
                        const deployResult = (yield postCompile_1.default(this.deployConfig, 'front')) || null;
                        log_1.default.success('[前端]静态文件部署结束');
                        return yield postDeploy_1.default(projectConfig, this.deployConfig, deployResult, this.git, 'front');
                    });
                    postDeployPromises.push(frontendPromise());
                }
                yield Promise.all(postDeployPromises);
                log_1.default.info('所有静态文件部署结束');
            }
            catch (error) {
                log_1.default.error(error);
            }
        });
    }
}
const wrapper = (taskKey, common) => {
    return new Deploy(taskKey, common);
};
exports.default = wrapper;
