"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const readDir_1 = require("../../../util/readDir");
const vmHandle_1 = require("./tasks/vmHandle");
const pipefile_1 = require("./tools/pipefile");
const postCompile = (deployConfig, deployType) => {
    const project = global.__PROJECT__;
    const promise = new Promise((resolve, reject) => {
        const files = readDir_1.default(deployConfig.build);
        const vmPromise = [], cssPromise = [], jsPromise = [], imgPromise = [];
        files.forEach((file) => {
            if (file.name.indexOf('.DS_Store') > -1) {
                return;
            }
            const relPath = path.relative(deployConfig.build, file.path);
            const fileType = relPath.split(path.sep)[0];
            switch (fileType) {
                case 'WEB-INF':
                    if (deployConfig.CONFIG.type.includes('vm') && deployConfig.backVMdir && deployType === 'back') {
                        const fileParseArr = relPath.split(path.sep);
                        const _fileName = fileParseArr[fileParseArr.length - 1];
                        let _path = '';
                        for (let i = 1; i < fileParseArr.length - 1; i++) {
                            _path += fileParseArr[i] + '/';
                        }
                        if (_path) {
                            _path = _path.substring(0, _path.length - 1);
                        }
                        if (_path === 'layout') {
                            vmPromise.push(vmHandle_1.default(file.path, path.resolve(deployConfig.backVMLayoutDir, _fileName), deployConfig));
                        }
                        else {
                            vmPromise.push(vmHandle_1.default(file.path, path.resolve(deployConfig.backVMTplDir, _path, _fileName), deployConfig));
                        }
                    }
                    break;
                case 'css':
                    if (deployConfig.CONFIG.type.includes('css')) {
                        if (/\.map$/.test(relPath)) {
                            return;
                        }
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && cssPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && cssPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                case 'js':
                    if (deployConfig.CONFIG.type.includes('js')) {
                        if (/\.map$/.test(relPath)) {
                            return;
                        }
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && jsPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && jsPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                case 'images':
                    if (deployConfig.CONFIG.type.includes('images')) {
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && imgPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && imgPromise.push(pipefile_1.default(file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                default:
                    break;
            }
        });
        resolve(Promise.all([
            Promise.all(vmPromise),
            Promise.all(cssPromise),
            Promise.all(jsPromise),
            Promise.all(imgPromise),
        ]));
    });
    return promise;
};
exports.default = postCompile;
