"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const iconv = require("iconv-lite");
const log_1 = require("../../../../util/log");
const mkdirs_1 = require("../../../../util/mkdirs");
const compare_1 = require("../tools/compare");
const vmHandle = (src, dest, config, handle) => {
    const promise = new Promise((resolve, reject) => {
        fs.readFile(src, (err, data) => {
            if (err) {
                reject(err);
                return;
            }
            const destDirName = path.dirname(dest);
            mkdirs_1.default(destDirName);
            let encodedSrcContent = data.toString();
            let destFileData, encodedDestContent = '';
            handle && (encodedSrcContent = handle(data.toString(), src));
            if (fs.existsSync(dest)) {
                destFileData = fs.readFileSync(dest);
                if (config.convertVMToGBK === true) {
                    const decodeDestCache = iconv.decode(destFileData, 'gbk');
                    encodedDestContent = iconv.encode(decodeDestCache, 'utf8').toString();
                }
                else {
                    encodedDestContent = destFileData.toString();
                }
            }
            let encodedContent = encodedSrcContent;
            if (config.convertVMToGBK === true) {
                const decodeSrcCache = iconv.decode(new Buffer(encodedSrcContent), 'utf-8');
                log_1.default.debug(`格式化 ${path.basename(src)} 为gbk`);
                encodedContent = iconv.encode(decodeSrcCache, 'gbk');
            }
            compare_1.default(encodedSrcContent, encodedDestContent).then((result) => {
                if (!result) {
                    log_1.default.debug(`文件有变动: ${path.basename(src)}`);
                    fs.writeFile(dest, encodedContent, null, (_err) => {
                        if (_err) {
                            reject(_err);
                            return;
                        }
                        log_1.default.success(`创建 后端 文件: ${path.basename(dest)}`);
                        resolve();
                    });
                }
                else {
                    resolve();
                }
            }).catch((_err) => {
                reject(err);
            });
        });
    });
    return promise;
};
const vmPostCompile = (file, dest, config) => {
    const promise = new Promise((resolve, reject) => {
        resolve(vmHandle(file, dest, config, config.vmHandle));
    });
    return promise;
};
exports.default = vmPostCompile;
