"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const crypto = require("crypto");
const log_1 = require("../../../../util/log");
const computeHash = (filename, algo) => {
    const chksum = crypto.createHash(algo);
    const promise = new Promise((resolve, reject) => {
        let s;
        if (fs.existsSync(filename)) {
            s = fs.createReadStream(filename);
            s.on('error', (err) => {
                reject(err);
            });
            s.on('data', (d) => {
                chksum.update(d);
            });
            s.on('end', () => {
                const d = chksum.digest('hex');
                resolve(d);
            });
        }
        else {
            chksum.update(filename);
            const d = chksum.digest('hex');
            resolve(d);
        }
    });
    return promise;
};
const compare = (file1, file2, algo = 'sha1') => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const file1hash = yield computeHash(file1, algo);
        const file2hash = yield computeHash(file2, algo);
        if (file1hash === file2hash) {
            return true;
        }
        else {
            let _file1 = file1, _file2 = file2;
            if (fs.existsSync(file1)) {
                _file1 = fs.readFileSync(file1, 'utf8');
            }
            if (fs.existsSync(file2)) {
                _file2 = fs.readFileSync(file2, 'utf8');
            }
            return _file1 === _file2;
        }
    }
    catch (error) {
        log_1.default.error(error.message);
        return false;
    }
});
exports.default = compare;
