"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const fs = require("fs");
const path = require("path");
const util = require("util");
const readline = require("readline");
const stream_1 = require("stream");
const compare_1 = require("./compare");
const mkdirs_1 = require("../../../../util/mkdirs");
const log_1 = require("../../../../util/log");
const now_1 = require("../../../../util/now");
const dophin_core_1 = require("dophin-core");
const GetGitStatus_1 = require("../../dependencies/GetGitStatus");
const result_1 = require("../../../compile/result");
const gitStatus = GetGitStatus_1.default();
const isLocalCommitContain = (id) => {
    for (const history of gitStatus.history) {
        if (history[0].substr(0, 5) === id.substr(0, 5)) {
            return true;
        }
    }
    return false;
};
const compareTime = (fileCommitTime) => {
    const firstHistory = gitStatus.history[gitStatus.history.length - 1];
    const firstHistoryCommitTime = new Date(firstHistory[2]).getTime();
    if (new Date(fileCommitTime).getTime() < firstHistoryCommitTime) {
        return true;
    }
    return false;
};
const isEntryModified = (filename) => __awaiter(void 0, void 0, void 0, function* () {
    if (path.extname(filename) !== '.js') {
        return true;
    }
    if (!Array.isArray(gitStatus.modified) || !gitStatus.modified.length) {
        return true;
    }
    const { rootPath = '' } = yield dophin_core_1.getRootConfig();
    const _jsModules = result_1.jsModules.get();
    const _modules = _jsModules[filename];
    const isGitHasJsModified = () => {
        for (const _mf of gitStatus.modified) {
            if (_mf.includes(`${global.__PROJECT__}${os.EOL}source${os.EOL}js`)) {
                return true;
            }
        }
        return false;
    };
    if (!isGitHasJsModified()) {
        return true;
    }
    if (_modules) {
        for (const m of _modules) {
            for (const _mf of gitStatus.modified) {
                if (m.includes(_mf)) {
                    log_1.default.info(`${filename} 模块有改动, 重新部署!`);
                    return true;
                }
            }
        }
        log_1.default.info(`${filename} 模块均无改动`);
        return false;
    }
    return true;
});
const ignoreAddFilenameVersion = (filename, projectConfig) => {
    if (projectConfig.version) {
        if (/bundle\.\w{5}\.js/.test(filename)) {
            return true;
        }
    }
    return false;
};
const AddNoteTransform = function (filetype, ver) {
    const that = this;
    if (!(this instanceof AddNoteTransform)) {
        return new AddNoteTransform(filetype);
    }
    stream_1.Transform.call(this, filetype);
    this._transformCount = 0;
    this._fileType = filetype;
    this._oldVersion = ver;
};
util.inherits(AddNoteTransform, stream_1.Transform);
AddNoteTransform.prototype._transform = function (data, encoding, callback) {
    if (this._transformCount === 0) {
        switch (this._fileType) {
            case 'js':
            case 'css':
                this.push(`/*ver:${gitStatus.version} - ${now_1.default()}*/\n`);
                break;
            default:
                break;
        }
    }
    this._transformCount++;
    callback(null, data);
};
AddNoteTransform.prototype._flush = function (callback) {
    callback();
};
const pipeFile = (src, dest, option, config) => {
    const promise = new Promise((resolve, reject) => {
        const readStream = fs.createReadStream(src);
        let writeStream, type, ver;
        if (option === false) {
            writeStream = fs.createWriteStream(dest);
            readStream.pipe(writeStream);
        }
        else {
            type = option.type;
            ver = option.ver || '';
            const commitHistoryReg = /^\/\*ver:(\w{5,10})\s.+(\d{4}\/\d{2}\/\d{2}.+)\*\//;
            const match = commitHistoryReg.exec(ver);
            if (commitHistoryReg.test(ver) && match !== null) {
                const historyId = match[1], commitTime = match[2];
                if (!isLocalCommitContain(historyId) && !compareTime(commitTime)) {
                    log_1.default.error(`${path.relative(config.build, src)} 版本高于本地, 您可能覆盖了其他人的文件! 请尝试先 pull 再部署!`);
                }
            }
            writeStream = fs.createWriteStream(dest);
            readStream.pipe(new AddNoteTransform(type, ver))
                .pipe(writeStream);
        }
        readStream.on('error', function (e) {
            reject(e);
        });
        writeStream.on('error', function (e) {
            reject(e);
        });
        writeStream.on('close', () => {
            if (dest.indexOf(config.backGitDir) > -1) {
                log_1.default.success(`创建 后端 文件: ${path.basename(dest)}`);
            }
            if (dest.indexOf(config.frontGitDir) > -1) {
                log_1.default.success(`创建 前端 文件: ${path.basename(dest)}`);
            }
            resolve(dest);
        });
    });
    return promise;
};
const copyFile = (filesrc, { destDir, relPath }, config) => __awaiter(void 0, void 0, void 0, function* () {
    if (!(yield isEntryModified(filesrc))) {
        return {
            path: relPath,
            type: 'constant',
        };
    }
    const projectConfig = yield dophin_core_1.getProjectConfig('');
    const filedestStr = path.resolve(destDir, relPath);
    const destDirName = path.dirname(filedestStr);
    mkdirs_1.default(destDirName);
    const type = path.extname(filesrc).split('.')[1];
    const basename = path.basename(filedestStr);
    const destFileName = path.basename(filedestStr, `.${type}`);
    let destFilePath = filedestStr;
    if ((type === 'js' || type === 'css') && projectConfig.version) {
        const historyFileReg = new RegExp(`${destFileName}(_[0-9]{6,8})?(_(.+))?.${type}`);
        const filesList = fs.readdirSync(destDirName).filter((file) => historyFileReg.test(file)).sort().reverse();
        filesList[0] && (destFilePath = path.join(destDirName, filesList[0]));
        log_1.default.debug(`找到目标目录最近一次部署的文件: ${path.relative(destDir, destFilePath)}`);
    }
    const isEnableAddDeployVersion = !!(projectConfig.version &&
        !(projectConfig.disableVersionFiles && projectConfig.disableVersionFiles.includes(basename)) &&
        !ignoreAddFilenameVersion(basename, projectConfig));
    if ((type === 'js' || type === 'css') && fs.existsSync(destFilePath)) {
        const verReg = /^\/\*ver:.+\*\//, verParReg = /^\/\*ver:.+\*\/\r?\n/;
        const _destOriginContent = fs.readFileSync(destFilePath, 'utf8');
        const _destContent = _destOriginContent.replace(verParReg, '');
        try {
            const compareResult = yield compare_1.default(filesrc, _destContent);
            let newDestFilePath = filedestStr;
            if (isEnableAddDeployVersion && projectConfig.version) {
                newDestFilePath = path.join(destDirName, `${destFileName}_${projectConfig.version.replace(/\./g, '_')}.${type}`);
            }
            if (!compareResult) {
                log_1.default.info(`文件有变动: ${basename}`);
                const readPromise = new Promise((resolve, reject) => {
                    const readDestStream = fs.createReadStream(destFilePath, {
                        encoding: 'utf8',
                    });
                    const readDestLine = readline.createInterface({
                        input: readDestStream,
                        terminal: false,
                    });
                    let _ver = '';
                    readDestLine.on('line', (line) => {
                        line = line.trim();
                        if (verReg.test(line)) {
                            _ver = line;
                        }
                        readDestLine.removeAllListeners('line');
                        readDestLine.close();
                    });
                    readDestLine.on('close', () => {
                        resolve(_ver);
                    });
                });
                const ver = yield readPromise;
                if (isEnableAddDeployVersion) {
                    if (projectConfig.addNoVersionFile && fs.existsSync(filedestStr)) {
                        if (/_\d_\d/.test(newDestFilePath)) {
                            log_1.default.warn(`已额外部署 ${relPath} 无版本号文件`);
                            yield pipeFile(filesrc, filedestStr, {
                                type,
                                ver,
                            }, config);
                        }
                    }
                    if (newDestFilePath === destFilePath) {
                        if (projectConfig.overwrite) {
                            log_1.default.error(`您覆盖了 ${path.basename(destFilePath)}，请提前修改版本号！`);
                            return {
                                path: path.relative(destDir, yield pipeFile(filesrc, newDestFilePath, {
                                    type,
                                    ver,
                                }, config)),
                                type: 'modified',
                            };
                        }
                        else {
                            log_1.default.error(`您即将覆盖 ${path.basename(destFilePath)}，海豚帮你取消了，请提前修改版本号！`);
                        }
                    }
                    else {
                        return {
                            path: path.relative(destDir, yield pipeFile(filesrc, newDestFilePath, {
                                type,
                                ver,
                            }, config)),
                            type: 'added',
                        };
                    }
                }
                else {
                    return {
                        path: path.relative(destDir, yield pipeFile(filesrc, destFilePath, {
                            type,
                            ver,
                        }, config)),
                        type: 'modified',
                    };
                }
            }
            else {
                return {
                    path: path.relative(destDir, destFilePath),
                    type: 'constant',
                };
            }
        }
        catch (error) {
            log_1.default.error(`复制文件错误: ${error.message}`);
        }
    }
    else {
        try {
            const compareResult = yield compare_1.default(filesrc, destFilePath);
            if (!compareResult) {
                log_1.default.debug(`文件有变动: ${path.basename(filesrc)}`);
                const _changeType = fs.existsSync(destFilePath) ? 'modified' : 'added';
                return {
                    path: path.relative(destDir, yield pipeFile(filesrc, destFilePath, {
                        type,
                    }, config)),
                    type: _changeType,
                };
            }
        }
        catch (error) {
            log_1.default.error(`复制文件错误: ${error.message}`);
        }
    }
    return {
        path: relPath,
        type: 'constant',
    };
});
exports.default = copyFile;
