"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const stylelint = require("stylelint");
const log_1 = require("../../util/log");
const format = (filepath, rootDir = '') => {
    const configFile = path.resolve(rootDir, '.stylelintrc');
    if (!fs.existsSync(configFile)) {
        log_1.default.error(`${rootDir} 下找不到 stylelint 配置文件(.stylelintrc)`);
        return;
    }
    if (fs.existsSync(filepath)) {
        stylelint.lint({
            files: filepath,
            configBasedir: path.resolve(__dirname, '../../'),
            configFile,
            fix: true,
            formatter: 'string',
        }).then((resultObject) => {
            log_1.default.success(`${path.basename(filepath)} 格式化结束`);
            if (resultObject.errored) {
                console.log(resultObject.output);
            }
        }).catch((err) => {
            log_1.default.error(err.message);
        });
    }
    else {
        log_1.default.error(`找不到文件: ${filepath}`);
    }
};
exports.default = format;
