"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const fs = require("fs");
const path = require("path");
const dophin_core_1 = require("dophin-core");
const initServer_1 = require("./initServer");
const compile_1 = require("../compile");
const log_1 = require("../../util/log");
class ServerBase extends dophin_core_1.DophinBaseClass {
    constructor(port, enableSSL) {
        super({ port, enableSSL });
    }
    beforeTaskRun() {
        const { rootPath = '' } = this.runtime;
        if (rootPath.indexOf('build') !== -1) {
            log_1.default.error(`检测到项目根目录路径${rootPath}包含'build'，这将导致本地服务异常，请立即修改！`);
        }
    }
    taskRunning() {
        const project = global.__PROJECT__;
        const { rootConfig, rootPath } = this.runtime;
        let { port, enableSSL } = this.params;
        log_1.default.success('开始编译并监听本地静态文件');
        compile_1.default('watchAll');
        const app = initServer_1.default(rootConfig, rootPath);
        const onError = (error) => {
            if (error.syscall !== 'listen') {
                throw error;
            }
            switch (error.code) {
                case 'EACCES':
                    log_1.default.error(`监听 ${port} 端口需要 root 权限, 尝试 sudo 重新运行`);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    log_1.default.error(`${port} 端口被占用`);
                    process.exit(2);
                    break;
                default:
                    throw error;
            }
        };
        const onListening = () => {
            log_1.default.success(`本地服务开始运行, 监听端口: ${port}`);
        };
        app.set('port', port);
        if (enableSSL) {
            port = 443;
            const sslOptions = {
                key: fs.readFileSync(path.resolve(__dirname, '../../ssl/private.key')),
                cert: fs.readFileSync(path.resolve(__dirname, '../../ssl/cert.pem')),
            };
            https.createServer(sslOptions, app).listen(port).on('error', onError).on('listening', onListening);
        }
        http.createServer(app).listen(port).on('error', onError).on('listening', onListening);
    }
}
exports.default = ServerBase;
