"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const log_1 = require("../../util/log");
const express = require("express");
const cookieParser = require("cookie-parser");
const bodyParser = require("body-parser");
const velocity_mjd_1 = require("velocity-mjd");
const ejs = require("ejs");
const proxyMid = require("http-proxy-middleware");
const initRoutes_1 = require("./initRoutes");
const rewrite_1 = require("./middleware/rewrite");
const mock_1 = require("./middleware/mock");
const mapStatic_1 = require("./middleware/mapStatic");
const initServer = (rootConfig, rootPath) => {
    const doProxy = rootConfig && rootConfig.proxy, doRewrite = rootConfig && rootConfig.rewrite;
    const app = express();
    const paths = [];
    const rootDir = rootPath + '';
    if (global.__EJS__ === true) {
        app.engine('ejs', (filePath, options, callback) => {
            const str = fs.readFileSync(filePath, 'utf8');
            const rendered = ejs.render(str, options);
            return callback(null, rendered);
        });
        app.set('view engine', 'ejs');
    }
    else {
        app.engine('vm', (filePath, options, callback) => {
            let targetVM = "";
            let projectName = "";
            log_1.default.success(`vm开始渲染: ${path.relative(rootDir, filePath)}`);
            if (filePath.indexOf('.vm') !== -1) {
                const content = fs.readFileSync(filePath, 'utf8');
                if (content.indexOf('$layout') !== -1) {
                    const reg = /\$layout(\s*)=(\s*)"(.*\.vm)"/;
                    const matchs = content.match(reg);
                    if (matchs && matchs.length > 0) {
                        const files = filePath.split('build');
                        const _projectSplitArr = filePath.split('project');
                        targetVM = _projectSplitArr[_projectSplitArr.length - 1];
                        projectName = targetVM.split(path.sep)[1] || targetVM.split('\\')[1];
                        if (options.__VMOPTIONS__ && options.__VMOPTIONS__.childProjectName) {
                            const childProjectName = options.__VMOPTIONS__.childProjectName;
                            filePath = `${files[0]}build/WEB-INF/${childProjectName}/layout/${matchs[3]}`;
                        }
                        else {
                            filePath = files[0] + 'build/WEB-INF/layout/' + matchs[3];
                        }
                    }
                }
            }
            log_1.default.debug(`vm root: ${path.resolve(rootDir, 'project')}`);
            log_1.default.debug(`vm layout: ${filePath}`);
            const engine = new velocity_mjd_1.Engine({
                encoding: 'utf-8',
                root: path.resolve(rootDir, 'project'),
                template: filePath,
                targetVM: "." + targetVM,
            });
            options.Double = {
                parseDouble: (str) => {
                    return parseFloat(str);
                },
                parseInt: (str) => {
                    return parseInt(str);
                },
                parseString: (str) => {
                    return str + "";
                },
            };
            options['utils'] = {
                tostring: (obj) => {
                    return JSON.stringify(obj);
                },
                checkTimes: (dateTime) => {
                    const patten = /[0-9]{1,2}:00-[0-9]{1,2}:00/g;
                    return patten.test(dateTime);
                },
            };
            const lnPath = path.resolve(rootDir, 'data/ln/m_cn.json');
            const lnPath2 = path.resolve(rootDir, 'data/ln/m_cn_' + projectName + '.json');
            if (fs.existsSync(lnPath2) || fs.existsSync(lnPath)) {
                let _data;
                if (fs.existsSync(lnPath2)) {
                    _data = fs.readFileSync(path.resolve(rootDir, lnPath2));
                }
                else {
                    _data = fs.readFileSync(path.resolve(rootDir, lnPath));
                }
                const lnData = Object.assign(JSON.parse(_data.toString()));
                log_1.default.debug('语言包获取成功');
                options["res"] = {
                    getText: (textKey) => {
                        return lnData[textKey];
                    },
                    insertText: function () {
                        const _arg = Array.prototype.slice.call(arguments);
                        const _key = _arg[0];
                        let _values = _arg.slice(1);
                        if (_values[0].indexOf('_') > -1) {
                            _values = _values[0].split('_');
                        }
                        const text = lnData[_key], res = text.replace(/\{(\d)\}/g, function (result, index) {
                            return _values[index] || '';
                        });
                        return res;
                    },
                };
            }
            else {
                log_1.default.warn(`项目语言包 ${lnPath} 不存在`);
            }
            options["murl"] = {
                splitStr: (str1, str2) => {
                    if (str1 && str2) {
                        if (str1.indexOf(str2) !== -1) {
                            return str1.replace(str2, "");
                        }
                    }
                },
                composite: (base, sid) => {
                    let url;
                    const paramStr = base.indexOf("?") !== -1 ? "&" : "?";
                    if (base && sid) {
                        url = base + paramStr + "sid=" + escape(sid.replace(new RegExp("[\'\"\\s]", "gm"), ""));
                    }
                    else {
                        url = '';
                    }
                    return url.toString();
                },
            };
            options.NumberTool = {
                format: function (fort, str) {
                    return str + '.00';
                },
            };
            const rendered = engine.render(options);
            log_1.default.debug('vm渲染成功');
            return callback(null, rendered);
        });
        app.set('view engine', 'vm');
    }
    paths.push(path.join(rootDir, './project'));
    app.set('views', paths);
    app.set('view cache', false);
    doProxy && doProxy.forEach((proxy) => {
        proxy.config.logLevel = 'warn';
        log_1.default.debug(`反代: ${proxy.route} => ${proxy.config.target}`);
        app.use(proxy.route, proxyMid(proxy.config));
    });
    app.use(rewrite_1.default(rootConfig, rootDir));
    app.use(bodyParser.json());
    app.use(bodyParser.urlencoded({
        extended: false,
    }));
    app.use(cookieParser());
    app.use('/data', mock_1.default(rootDir));
    const projectDir = path.resolve(rootDir, 'project'), projectChildren = fs.readdirSync(projectDir);
    projectChildren.forEach((proj) => {
        if (proj.indexOf('.') !== 0) {
            const projectConfigPath = path.resolve(projectDir, proj, 'config.js');
            if (fs.existsSync(projectConfigPath)) {
                log_1.default.debug(`静态资源托管 ${proj}`);
                app.use(`/${proj}`, express.static(path.join(rootDir, `project/${proj}/build`)));
                app.use(`/static/${proj}`, express.static(path.join(rootDir, `project/${proj}/build`)));
                if (require(projectConfigPath).childProjects) {
                    const _childProjects = require(projectConfigPath).childProjects;
                    for (const _childP in _childProjects) {
                        app.use(`/${proj}/${_childP}`, express.static(path.join(rootDir, `project/${proj}/build`)));
                    }
                }
            }
        }
    });
    app.use(`/common/js`, express.static(path.join(rootDir, `project/common/source/js-lab`)));
    app.use(`/common/lib`, express.static(path.join(rootDir, `project/common/lib`)));
    app.use(initRoutes_1.default(rootPath));
    app.use(mapStatic_1.default(global.__PROJECT__, rootPath));
    app.use((req, res, next) => {
        const err = new Error(' O(∩_∩)O~ dophin找不到这个文件或者无法mock这个请求~~');
        err.status = 404;
        next(err);
    });
    app.use((err, req, res, next) => {
        res.status(err.status || 500);
        res.writeHead(200, {
            'Content-Language': 'zh-CN',
            'Content-Type': 'text/html;charset=UTF-8',
            'Cache-Control': 'max-age=0',
            'Server': 'dophin',
            'X-Powered-By': 'dophin',
        });
        res.end(`<h1>错误码：${err.status || 500}</h1>
        <h2 style="color:red">${err.message}</h2>
        <h2>请求方式：${req.method}</h2>
        <h2>请求路径：${req.url}</h2>
        <h3>错误stack：</h3>
        <div>${err.stack}</div>`);
    });
    return app;
};
exports.default = initServer;
