#!/usr/bin/env node
"use strict";
exports.__esModule = true;
var program = require("commander");
var chalk_1 = require("chalk");
var server_1 = require("../lib/server");
program
    .usage('[project] [option]')
    .option('-p, --port [port]', '设置本地服务端口号')
    .option('-s, --ssl', '是否监听本地 443 端口 [开启 https 服务]')
    .option('-e, --env [env]', '设置 NODE_ENV 环境变量 production/testing/development, 首字母也ok')
    .action(function (project) {
    switch (program.env) {
        case 'd':
        case 'development':
            process.env.NODE_ENV = 'development';
            break;
        case 't':
        case 'testing':
            process.env.NODE_ENV = 'testing';
            break;
        case 'p':
        case 'production':
            process.env.NODE_ENV = 'production';
            break;
        default:
            process.env.NODE_ENV = 'development';
            break;
    }
    // 设置项目名
    global.__PROJECT__ = project;
    server_1["default"](Number(program.port || 80), program.ssl);
});
program.on('--help', function () {
    console.log('\n  举个栗子:\n');
    console.log(chalk_1["default"].gray('    # 启动本地项目 example 服务'));
    console.log('    $ dop server example  或者简写  dop s example\n');
    console.log(chalk_1["default"].gray('    # 自定义端口和监听443端口(https)'));
    console.log('    $ dop server example --port 3000       或者  dop server example -p 3000');
    console.log('    $ dop server example --ssl             或者  dop server example -s');
    console.log('    $ dop server example --env production  或者  dop s example -e d\n');
});
(function activeHelp() {
    program.parse(process.argv);
    if (program.args.length < 1) {
        return program.help();
    }
})();
