"use strict";
/**
 * webpack的plugin，用于合并css文件
 */
exports.__esModule = true;
var fs = require("fs");
var webpack_sources_1 = require("webpack-sources");
var concatAndOutput = function (baseCss, concatCss, output) {
    var concatStr = new webpack_sources_1.ConcatSource(baseCss, concatCss);
    output && fs.writeFileSync(output, concatStr.source(), 'utf8');
};
var ConcatCss = /** @class */ (function () {
    function ConcatCss(options) {
        if (options === void 0) { options = {}; }
        // 插件传参
        this.options = {};
        // 记录webpack的chunks历史hash值
        this.chunkVersions = {};
        Object.assign(this.options, options);
    }
    ConcatCss.prototype.apply = function (compiler) {
        var _this = this;
        var _a = this.options, _b = _a.entryName, entryName = _b === void 0 ? '' : _b, sourceFile = _a.sourceFile, baseCss = _a.baseCss, output = _a.output;
        compiler.plugin("compilation", function (compilation) {
            compilation.plugin("optimize-chunk-assets", function (chunks, callback) {
                if (!sourceFile && !baseCss && !output) {
                    callback();
                    return;
                }
                // 计算变动过的chunks
                var changedChunks = chunks.filter(function (chunk) {
                    var oldVersion = _this.chunkVersions[chunk.name];
                    _this.chunkVersions[chunk.name] = chunk.hash;
                    return chunk.hash !== oldVersion;
                });
                // 寻找包含需要分离css的chunk
                var containCssChunk = chunks.filter(function (chunk) {
                    return chunk.files.includes(entryName);
                }).map(function (chunk) {
                    return chunk.name;
                });
                for (var _i = 0, changedChunks_1 = changedChunks; _i < changedChunks_1.length; _i++) {
                    var chunk = changedChunks_1[_i];
                    // 如果变动的chunk.name在containCssChunk中，则执行concat
                    if (containCssChunk.includes(chunk.name)) {
                        // TODO: 与css同时打包时的可能冲突问题
                        // 如果传入字符串，则直接拼接字符串
                        if (baseCss) {
                            concatAndOutput(baseCss, compilation.assets[entryName], output);
                            callback();
                            return;
                        }
                        // 如果传入css文件路径，则调用postcss打包该css文件，并将结果concat到输出css
                        // if (sourceFile && fs.existsSync(sourceFile)) {
                        //     (compileCss(sourceFile, 'compile', true) as Promise<string>).then(([css]: string) => {
                        //         concatAndOutput(css, compilation.assets[entryName], output);
                        //         callback();
                        //     });
                        // }
                        callback();
                        return;
                    }
                }
                callback();
            });
        });
    };
    return ConcatCss;
}());
exports["default"] = ConcatCss;
