"use strict";
/**
 * postcss插件配置
 */
exports.__esModule = true;
var path = require("path");
var postImport = require("postcss-import");
var stylelint = require("stylelint");
var cssnext = require("postcss-cssnext");
var cssmixins = require("postcss-mixins");
var cssnano = require("cssnano");
var px2rem = require("postcss-pxtorem");
var postcssFlexbugsFixes = require("postcss-flexbugs-fixes");
// 获取项目配置
var dophin_core_1 = require("dophin-core");
// 做个缓存，不用每次都计算
var pluginCache;
/**
 * 获取postcss插件配置
 * @param {object} { rootConfig, rootPath } - 根环境
 * @param {string} filePath - 打包文件路径
 * @returns {promise}
 */
var getPlugins = function (_a, filePath) {
    var rootConfig = _a.rootConfig, rootPath = _a.rootPath, projectConfig = _a.projectConfig;
    if (pluginCache) {
        return pluginCache;
    }
    // 这俩取值优先级需要将sass置前
    var cssConfig = projectConfig.sass || projectConfig.css;
    // 初始化px转rem配置
    var px2remConfig = {
        rootValue: (cssConfig.rem.rootValue) || ((16 * cssConfig.rem.psdWidth) / 375) || 32,
        unitPrecision: 4,
        propList: ['*'],
        selectorBlackList: [],
        replace: true,
        mediaQuery: false,
        minPixelValue: 0
    };
    // dophin运行目录
    var dophinRootPath = path.resolve(__dirname, "../../../");
    var plugins = [
        postImport,
        cssmixins,
        stylelint({
            fix: true,
            configFile: path.resolve(rootPath, '.stylelintrc'),
            configBasedir: dophinRootPath,
            codeFilename: path.basename(filePath || '')
        }),
        postcssFlexbugsFixes,
        cssnext({
            browsers: (rootConfig && rootConfig.browsers) || dophin_core_1.constants.browsers,
            warnForDuplicates: false
        }),
    ];
    // 是否将px转为rem
    if (cssConfig.rem && cssConfig.rem.need) {
        plugins.push(px2rem(px2remConfig));
    }
    // 生产环境压缩css
    if (process.env.NODE_ENV === 'production') {
        plugins.push(cssnano({
            preset: [
                "default",
                { cssDeclarationSorter: false },
            ]
        }));
    }
    pluginCache = plugins;
    return plugins;
};
exports["default"] = getPlugins;
