"use strict";
/**
 * 获取webpack配置
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var webpack = require("webpack");
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var getBabelConfig_1 = require("./getBabelConfig");
var log_1 = require("../../../util/log");
// 获取postcss插件方法
var getPostcssPlugins_1 = require("./getPostcssPlugins");
var getConfig = function (_a) {
    var rootConfig = _a.rootConfig, rootPath = _a.rootPath, projectConfig = _a.projectConfig;
    // 获取项目根目录
    var rootDir = rootPath + '';
    // dophin根目录和runtime组件目录
    var dophinRootPath = path.resolve(__dirname, "../../../");
    // 获取子项目配置
    var extractCss = projectConfig.extractCss;
    // 计算所有公用组件别名 webpack.alias
    var _alias = {}, commonDir = [
        path.resolve(rootDir, 'project/common/components'),
        path.resolve(rootDir, 'project/common/plugin'),
    ];
    commonDir.forEach(function (_dir) {
        if (!fs.existsSync(_dir)) {
            return;
        }
        var files = fs.readdirSync(_dir);
        log_1["default"].debug("\u8BFB\u53D6 project/commmon/" + path.basename(_dir) + " \u5185 webpack alias");
        files.forEach(function (file) {
            if (file.indexOf('.') !== -1) {
                file.split('.')[0] && (_alias[file.split('.')[0]] = path.resolve(_dir, file.split('.')[0]));
            }
            else {
                _alias[file] = path.resolve(_dir, file);
            }
        });
    });
    // 计算各子项目alias，规则: @project
    var childrenProjectsDir = path.resolve(rootPath, 'project');
    if (fs.existsSync(childrenProjectsDir)) {
        var childrenProjects = fs.readdirSync(childrenProjectsDir);
        childrenProjects.forEach(function (project) {
            if (fs.existsSync(path.resolve(childrenProjectsDir, project, 'config.js'))) {
                _alias["@" + project] = path.resolve(childrenProjectsDir, project, 'source', 'js');
            }
        });
    }
    // 依据 NODE_ENV 环境变量确定部分配置参数 webpack.plugins/webpack.devtool
    var webpackPlugins = [];
    var webpackDevtool = false;
    // 新增webpack环境插件
    webpackPlugins.push(new webpack.DefinePlugin({
        'process.env': {
            NODE_ENV: JSON.stringify(process.env.NODE_ENV)
        }
    }));
    // 生产环境压缩js，开发环境添加devtool
    if (process.env.NODE_ENV === 'production') {
        webpackDevtool = false;
        webpackPlugins.push(new webpack.optimize.UglifyJsPlugin());
    }
    else if (process.env.NODE_ENV === 'development') {
        webpackDevtool = 'cheap-module-eval-source-map';
    }
    else if (process.env.NODE_ENV === 'testing') {
        webpackDevtool = 'cheap-module-eval-source-map';
    }
    // webpack3 特性 scope hoisting
    // webpackPlugins.push(new webpack.optimize.ModuleConcatenationPlugin());
    // babel-loader配置
    var babelLoaderOptions = __assign({}, getBabelConfig_1.getBabelConfig(rootConfig), { cacheDirectory: true });
    // css的loader配置
    var cssLoaders = [
        {
            loader: 'css-loader',
            options: { importLoaders: 1 }
        },
        {
            loader: 'postcss-loader',
            options: {
                ident: 'postcss',
                plugins: getPostcssPlugins_1["default"]({ rootConfig: rootConfig, rootPath: rootPath, projectConfig: projectConfig })
            }
        },
    ];
    // 获取tsconfig.json位置，优先获取子项目内tsconfig
    var tsconfigPath = path.resolve(rootDir, "project/" + global.__PROJECT__ + "/tsconfig.json");
    if (!fs.existsSync(tsconfigPath)) {
        tsconfigPath = 'tsconfig.json';
    }
    var config = {
        context: dophinRootPath,
        module: {
            // 注意【test: /\.css$/】在getDefaultConfig.js 内配置
            rules: [
                {
                    test: /\.bundle\.js?$/,
                    use: [
                        {
                            loader: 'bundle-loader?lazy',
                            options: {
                                lazy: true,
                                name: '[name]'
                            }
                        },
                    ],
                    exclude: /node_modules/
                },
                {
                    test: /\.js$/,
                    loader: 'babel-loader',
                    exclude: /node_modules/,
                    options: babelLoaderOptions
                },
                {
                    test: /\.art$/,
                    use: [
                        {
                            loader: 'babel-loader',
                            options: babelLoaderOptions
                        },
                        {
                            loader: 'art-template-loader',
                            options: {
                                loader: require('art-template-loader'),
                                imports: path.resolve(rootDir, 'node_modules', 'art-template/lib/runtime')
                            }
                        },
                    ]
                },
                {
                    test: /\.jsx$/,
                    loader: 'babel-loader',
                    exclude: /node_modules/,
                    options: babelLoaderOptions
                },
                {
                    test: /\.bundle\.tsx?$/,
                    use: [{
                            loader: 'bundle-loader?lazy',
                            options: {
                                lazy: true,
                                name: '[name]'
                            }
                        }],
                    exclude: /node_modules/
                },
                {
                    test: /\.tsx?$/,
                    loader: 'awesome-typescript-loader',
                    exclude: /node_modules/,
                    options: {
                        configFileName: tsconfigPath
                    }
                },
                // 如果配置了分离css，则需要对loader做点修改
                {
                    test: /\.css$/,
                    use: (extractCss ?
                        ExtractTextPlugin.extract({
                            fallback: 'style-loader',
                            use: cssLoaders
                        }) : ['style-loader'].concat(cssLoaders))
                },
                {
                    test: /\.scss$/,
                    use: (extractCss ?
                        ExtractTextPlugin.extract({
                            fallback: 'style-loader',
                            use: cssLoaders.concat(['sass-loader'])
                        }) : ['style-loader'].concat(cssLoaders, ['sass-loader']))
                }, {
                    test: /\.(png|jpg|gif)$/,
                    use: [{
                            loader: 'url-loader',
                            options: {
                                limit: 8192
                            }
                        }]
                },
            ]
        },
        devtool: webpackDevtool,
        plugins: webpackPlugins,
        resolve: {
            alias: _alias,
            extensions: ['.js', '.ts', '.tsx', '.jsx', '.css', '.scss']
        }
    };
    return config;
};
exports["default"] = getConfig;
