"use strict";
/**
 * 初始化并校验deploy项目配置
 */
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var log_1 = require("../../../util/log");
var configCache;
var initConfig = function (taskKey, rootPath, rootConfig) {
    if (configCache) {
        return configCache;
    }
    var config = rootConfig && rootConfig.deploy[taskKey];
    var rootDir = rootPath + '';
    if (!config) {
        log_1["default"].error("\u68C0\u6D4B\u4E0D\u5230 deploy." + taskKey + " \u7684\u914D\u7F6E(do.config.js\u5185\u6DFB\u52A0)");
        return null;
    }
    var project = config.projectName || taskKey;
    global.__PROJECT__ = project;
    // 给部署目录路径做一个映射
    config.projNameInStaticDirMap = config.projNameInStaticDirMap || {};
    var _a = [config.projNameInStaticDirMap.back, config.projNameInStaticDirMap.front], _b = _a[0], projectNameInBackStatic = _b === void 0 ? project : _b, _c = _a[1], projectNameInFrontStatic = _c === void 0 ? project : _c;
    // 处理一些配置, 尽量做一些容错
    config.javaWebName = config['javaWebName'] || project + "-web";
    // 本地项目构建前目录
    config.source = path.resolve(rootDir, "project/" + project + "/source");
    // 本地项目构建后目录
    config.build = path.resolve(rootDir, "project/" + project + "/build");
    if (config.CONFIG.git.includes('back')) {
        if (!config.backGitDir) {
            log_1["default"].error("\u60A8\u6CA1\u6709\u914D\u7F6E backGitDir \u53C2\u6570");
            return null;
        }
        var backGitDir = path.resolve(rootDir, config.backGitDir);
        if (!fs.existsSync(backGitDir)) {
            log_1["default"].error("backGitDir: \u4E0D\u5B58\u5728\u76EE\u5F55: " + backGitDir);
            return null;
        }
        config.backGitDir = backGitDir;
        // 后端git静态文件目录
        config.backStaticDir = path.resolve(config.backGitDir, config.javaWebName + "/src/main/webapp", config['backStaticDirHasProjName'] === false ? '' : projectNameInBackStatic);
        // 后端vm文件目录
        config.backVMdir = path.resolve(config.backGitDir, config.javaWebName + "/src/main/webapp", config['vmDir'] || 'WEB-INF/vm');
        config.backVMLayoutDir = path.resolve(config.backGitDir, config.javaWebName + "/src/main/webapp", config['vmLayoutDir'] || 'WEB-INF/vm/layout');
        config.backVMTplDir = path.resolve(config.backGitDir, config.javaWebName + "/src/main/webapp", config['vmTplDir'] || "WEB-INF/vm/m/" + project);
    }
    if (config.CONFIG.git.includes('front')) {
        if (!config.frontGitDir) {
            log_1["default"].error("\u60A8\u6CA1\u6709\u914D\u7F6E frontGitDir \u53C2\u6570");
        }
        var frontGitDir = path.resolve(rootDir, config.frontGitDir);
        if (!fs.existsSync(frontGitDir)) {
            log_1["default"].error("frontGitDir: \u4E0D\u5B58\u5728\u76EE\u5F55: " + frontGitDir);
            return null;
        }
        config.frontGitDir = frontGitDir;
        // 前端git静态文件目录
        config.frontStaticDir = path.resolve(config.frontGitDir, 'src/main/webapp', projectNameInFrontStatic);
    }
    configCache = config;
    return config;
};
exports["default"] = initConfig;
