"use strict";
/**
 * 获取本地开发目录git信息
 */
exports.__esModule = true;
var os = require("os");
var child_process_1 = require("child_process");
var log_1 = require("../../../util/log");
var gitCache;
var getGitStatus = function () {
    if (gitCache) {
        return gitCache;
    }
    var _git = {
        version: '',
        dirty: 0,
        history: [],
        modified: []
    };
    var commitIdLog = child_process_1.execSync('git rev-parse --short HEAD', {
        encoding: 'utf8'
    });
    var statusLog = child_process_1.execSync('git status -s', {
        encoding: 'utf8'
    });
    var commitHistoryLog = child_process_1.execSync('git log --pretty=format:"%h -#- %an -#- %ai -#- %s" -100', {
        encoding: 'utf8'
    });
    var EOL = os.EOL;
    statusLog.split(EOL).filter(function (data) {
        return !!data;
    }).forEach(function (logs) {
        logs = logs.replace(/^\s?/, '');
        var _status = logs.split(/\s{1,2}/);
        if (!_git[_status[0]]) {
            _git[_status[0]] = [];
        }
        _git[_status[0]].push(_status[1]);
        _git['modified'].push(_status[1]);
    });
    // 计算已修改但未提交的文件个数
    for (var type in _git) {
        if (type !== 'version' && type !== 'dirty') {
            _git.dirty += _git[type].length;
        }
    }
    // 计算最近提交记录
    commitHistoryLog.split(EOL).filter(function (data) {
        return !!data;
    }).forEach(function (logs) {
        var historyInfoArray = logs.split(/\s-#-\s/);
        _git.history.push(historyInfoArray);
    });
    _git.version = commitIdLog.replace('\n', '');
    log_1["default"].info("\u5F53\u524D git commit: " + _git.version);
    _git.dirty && log_1["default"].warn("\u5F53\u524D git \u8FD8\u6709 " + _git.dirty + " \u4E2A\u6587\u4EF6\u6CA1\u6709\u63D0\u4EA4, \u8BF7\u5C3D\u5FEBcommit\u5E76push, \u5426\u5219\u60A8\u53D1\u5E03\u7684\u4EE3\u7801\u53EF\u80FD\u88AB\u522B\u4EBA\u8986\u76D6");
    gitCache = _git;
    return _git;
};
exports["default"] = getGitStatus;
