"use strict";
exports.__esModule = true;
var path = require("path");
var readDir_1 = require("../../../util/readDir");
var vmHandle_1 = require("./tasks/vmHandle");
var pipefile_1 = require("./tools/pipefile");
var postCompile = function (deployConfig, deployType) {
    var project = global.__PROJECT__;
    var promise = new Promise(function (resolve, reject) {
        var files = readDir_1["default"](deployConfig.build);
        var vmPromise = [], cssPromise = [], jsPromise = [], imgPromise = [];
        files.forEach(function (file) {
            // mac隐藏文件忽略
            if (file.name.indexOf('.DS_Store') > -1) {
                return;
            }
            var relPath = path.relative(deployConfig.build, file.path);
            var fileType = relPath.split(path.sep)[0];
            switch (fileType) {
                case 'WEB-INF':
                    // vm文件处理 => 拷贝到后端预发git文件夹
                    if (deployConfig.CONFIG.type.includes('vm') && deployConfig.backVMdir && deployType === 'back') {
                        // 处理vm文件的相关参数
                        var fileParseArr = relPath.split(path.sep);
                        var _fileName = fileParseArr[fileParseArr.length - 1];
                        var _path = '';
                        for (var i = 1; i < fileParseArr.length - 1; i++) {
                            _path += fileParseArr[i] + '/';
                        }
                        if (_path) {
                            _path = _path.substring(0, _path.length - 1);
                        }
                        if (_path === 'layout') {
                            // layout处理
                            vmPromise.push(vmHandle_1["default"](file.path, path.resolve(deployConfig.backVMLayoutDir, _fileName), deployConfig));
                        }
                        else {
                            // 处理其他vm文件, 直接放到 proj 文件夹
                            vmPromise.push(vmHandle_1["default"](file.path, path.resolve(deployConfig.backVMTplDir, _path, _fileName), deployConfig));
                        }
                    }
                    break;
                case 'css':
                    // css文件处理 => 拷贝到后端 + 前端文件夹
                    if (deployConfig.CONFIG.type.includes('css')) {
                        if (/\.map$/.test(relPath)) {
                            // .map文件忽略
                            return;
                        }
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && cssPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && cssPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                case 'js':
                    // js文件处理 => 拷贝到后端 + 前端文件夹
                    if (deployConfig.CONFIG.type.includes('js')) {
                        if (/\.map$/.test(relPath)) {
                            // .map文件忽略
                            return;
                        }
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && jsPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && jsPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                case 'images':
                    // images文件处理 => 拷贝到后端 + 前端文件夹
                    if (deployConfig.CONFIG.type.includes('images')) {
                        deployType === 'back'
                            && deployConfig.backStaticDir
                            && imgPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.backStaticDir, relPath: relPath }, deployConfig));
                        deployType === 'front'
                            && deployConfig.frontStaticDir
                            && imgPromise.push(pipefile_1["default"](file.path, { destDir: deployConfig.frontStaticDir, relPath: relPath }, deployConfig));
                    }
                    break;
                default:
                    break;
            }
        });
        resolve(Promise.all([
            Promise.all(vmPromise),
            Promise.all(cssPromise),
            Promise.all(jsPromise),
            Promise.all(imgPromise),
        ]));
    });
    return promise;
};
exports["default"] = postCompile;
