"use strict";
exports.__esModule = true;
/**
 * 编译后处理方法
 */
var fs = require("fs");
var path = require("path");
var iconv = require("iconv-lite");
var log_1 = require("../../../../util/log");
var mkdirs_1 = require("../../../../util/mkdirs");
var compare_1 = require("../tools/compare");
var vmHandle = function (src, dest, config, handle) {
    var promise = new Promise(function (resolve, reject) {
        fs.readFile(src, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
            // 获取复制文件的目标目录
            var destDirName = path.dirname(dest);
            // 如果目标目录不存在,那就创建一个
            mkdirs_1["default"](destDirName);
            // 源文件字符串
            var encodedSrcContent = data.toString();
            // 目标文件
            var destFileData, encodedDestContent = '';
            // deploy内vmHandle处理
            handle && (encodedSrcContent = handle(data.toString(), src));
            if (fs.existsSync(dest)) {
                destFileData = fs.readFileSync(dest);
                if (config.convertVMToGBK === true) {
                    // 目标内容转gbk
                    var decodeDestCache = iconv.decode(destFileData, 'gbk');
                    encodedDestContent = iconv.encode(decodeDestCache, 'utf8').toString();
                }
                else {
                    encodedDestContent = destFileData.toString();
                }
            }
            var encodedContent = encodedSrcContent;
            if (config.convertVMToGBK === true) {
                // 新内容转gbk
                var decodeSrcCache = iconv.decode(new Buffer(encodedSrcContent), 'utf-8');
                log_1["default"].debug("\u683C\u5F0F\u5316 " + path.basename(src) + " \u4E3Agbk");
                encodedContent = iconv.encode(decodeSrcCache, 'gbk');
            }
            compare_1["default"](encodedSrcContent, encodedDestContent).then(function (result) {
                if (!result) {
                    log_1["default"].debug("\u6587\u4EF6\u6709\u53D8\u52A8: " + path.basename(src));
                    fs.writeFile(dest, encodedContent, null, function (_err) {
                        if (_err) {
                            reject(_err);
                            return;
                        }
                        log_1["default"].success("\u521B\u5EFA \u540E\u7AEF \u6587\u4EF6: " + path.basename(dest));
                        resolve();
                    });
                }
                else {
                    resolve();
                }
            })["catch"](function (_err) {
                reject(err);
            });
        });
    });
    return promise;
};
var vmPostCompile = function (file, dest, config) {
    // 构建后处理vm并部署
    var promise = new Promise(function (resolve, reject) {
        resolve(vmHandle(file, dest, config, config.vmHandle));
    });
    return promise;
};
exports["default"] = vmPostCompile;
