"use strict";
/**
 * 文件全部部署之后的操作
 */
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var now_1 = require("../../../util/now");
var log_1 = require("../../../util/log");
var readDir_1 = require("../../../util/readDir");
var postDeploy = function (projectConfig, config, deployResult, git, deployType) {
    var deployTypeName = deployType === 'back' ? '后端' : '前端';
    return new Promise(function (resolve, reject) {
        if (deployResult === null) {
            log_1["default"].error('好像发生了什么错误，并没有部署什么静态文件~');
            return;
        }
        // 删除vm记录
        deployResult.splice(0, 1);
        // 计算部署文件数量
        var deployedFilesCount = 0, changedFilesCount = 0;
        deployResult.forEach(function (deployTypeArr, index) {
            deployedFilesCount += deployTypeArr.length || 0;
            if (Array.isArray(deployTypeArr) && deployTypeArr.length > 0) {
                deployTypeArr.forEach(function (file) {
                    // 带版本号的文件添加一个不带版本号文件记录
                    var versionReg = /^(.+)(_\d){2,5}(\.\w{2,5})$/i;
                    if (projectConfig.addNoVersionFile && file && versionReg.test(file.path)) {
                        var newFilePath = file.path.replace(/(_\d){2,5}/, '');
                        deployResult[index].push({
                            path: newFilePath,
                            type: file.type
                        });
                    }
                    // 统计变动文件数量
                    if (file && file.type !== 'constant') {
                        changedFilesCount++;
                    }
                });
            }
        });
        if (deployedFilesCount === 0 || changedFilesCount === 0) {
            log_1["default"].error('并没有部署什么静态文件，你怕不是在搞我~');
            return;
        }
        if (!projectConfig.version) {
            resolve();
            return;
        }
        // 记录部署日志，并将日志写入部署目录
        var date = now_1["default"]('yyyymmdd');
        var version = projectConfig.version || '0.0.0';
        // 本地部署日志内容
        var deployLog = {
            date: date,
            version: version,
            commit: git.version,
            result: deployResult
        };
        // 日志内容数组
        var deployLogArr = [];
        var logFileName = '.dophinlog.json';
        var deployPath = '', logPath = '';
        if (deployType === 'back') {
            deployPath = config.backStaticDir;
        }
        if (deployType === 'front') {
            deployPath = config.frontStaticDir;
        }
        logPath = path.resolve(deployPath, logFileName);
        if (!logPath) {
            return;
        }
        // 获取历史部署日志
        if (fs.existsSync(logPath)) {
            deployLogArr = require(logPath);
        }
        // 添加本次部署日志
        deployLogArr.unshift(deployLog);
        // 获取所有version
        log_1["default"].info("[" + deployTypeName + "]\u5F00\u59CB\u5408\u5E76dophinlog...");
        var versions = new Set(deployLogArr.map(function (_log) { return _log.version; }));
        // 对deployLogArr依据version去重
        deployLogArr = deployLogArr.filter(function (_log) {
            if (versions.has(_log.version)) {
                versions["delete"](_log.version);
                return true;
            }
            else {
                return false;
            }
        });
        // 如果部署日志超过2个，才会清理无用文件
        if (deployLogArr.length > 2 && config.deleteInvalidFiles) {
            // 开始清理无用文件
            log_1["default"].info("[" + deployTypeName + "]\u5F00\u59CB\u6839\u636Edophinlog\u6E05\u7406\u65E0\u7528\u6587\u4EF6...");
            // 默认有效版本为99个
            var validVersions_1 = projectConfig.deployValidVersions || 99;
            // 获取所有有用文件
            var usefulFiles_1 = new Set();
            deployLogArr.forEach(function (_log, index) {
                // 如果没有超过有效版本数，则认为是有用的文件
                if (index < validVersions_1) {
                    for (var _i = 0, _a = _log.result; _i < _a.length; _i++) {
                        var files = _a[_i];
                        for (var _b = 0, files_1 = files; _b < files_1.length; _b++) {
                            var file = files_1[_b];
                            usefulFiles_1.add(path.resolve(deployPath, file.path));
                        }
                    }
                }
            });
            // 获取待部署目录内所有文件
            var filesInDeployPath = readDir_1["default"](deployPath);
            // 删除无用文件
            var deletedFiles_1 = [];
            filesInDeployPath.forEach(function (file) {
                if (!usefulFiles_1.has(file.path) &&
                    /\/(js|css|images)\b/.test(file.path) &&
                    /\.(js|css|png|jpg|gif|jpeg|)$/i.test(file.name)) {
                    fs.unlinkSync(file.path);
                    deletedFiles_1.push(file.name);
                }
            });
            if (deletedFiles_1.length) {
                log_1["default"].warn("[" + deployTypeName + "]\u5220\u9664\u6587\u4EF6[" + deletedFiles_1.length + "]\u4E2A: " + deletedFiles_1.join(', '));
            }
            else {
                log_1["default"].info("[" + deployTypeName + "]\u672C\u6B21\u66F4\u65B0\u4E3A\u589E\u91CF\u66F4\u65B0\uFF0C\u672A\u5220\u9664\u4EFB\u4F55\u6587\u4EF6");
            }
        }
        var logFileContent = JSON.stringify(deployLogArr) + "\n";
        // 记录部署日志到静态文件根目录
        fs.writeFile(logPath, logFileContent, { encoding: 'utf8' }, function (err) {
            if (err) {
                reject(err);
            }
            else {
                if (!config.deleteInvalidFiles) {
                    log_1["default"].warn('[危险,谨慎操作]dophin部署过程可能产生垃圾文件(e.g. chunk[hash])，可在do.config.js添加项目deploy参数deleteInvalidFiles以自动删除垃圾文件。');
                }
                resolve();
            }
        });
    });
};
exports["default"] = postDeploy;
