"use strict";
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var express = require("express");
var log_1 = require("../../util/log");
var initRoutes = function (rootPath) {
    var router = express.Router();
    /* GET home page.
     * 路由规则：
     * ip:<端口>/项目名称(project文件下项目工程目录名)／文件名/data数据文件名
     * 根据地址信息，获取初始化配置数据和初始化显示数据
     * JSON数据保存位置在data文件夹下面
     * data／config下面保存配置数据，跟请求的pageName相同
     * 初始化显示数据与请求的action相同
     */
    router.get('/(([a-zA-Z0-9-]+/)+)?:pageName/:dataName', function (req, res, next) {
        var rootDir = rootPath + '';
        var projectName, dataPath = '', filePath = '', childProjectName = '', // 完成的项目名称,包含子项目
        configPath = '';
        var pageName = req.params.pageName, dataName = req.params.dataName, _path = req.path;
        if (dataName.indexOf('.') !== -1 || pageName === 'data' || pageName === 'mock') {
            // 如果是静态文件(dataName带个点)
            next();
            return;
        }
        log_1["default"].debug("\u8BF7\u6C42\u8DEF\u5F84 " + _path);
        var paths = _path.split('/'), len = paths.length;
        var _filePathStartIndex = 2;
        projectName = paths[1];
        log_1["default"].debug("\u9879\u76EE\u540D\u79F0 " + projectName);
        // 读取子项目配置
        var projectConfig = {};
        var projectPath = path.resolve(rootDir, 'project/', projectName), projectConfigPath = path.resolve(projectPath, 'config.js');
        if (fs.existsSync(projectConfigPath)) {
            projectConfig = require(projectConfigPath);
        }
        // 如果当前 projectName 项目存在子项目(config.js中存在 childProjects{Array} 配置), 目前只做一级子项目配置
        var _childProjects = projectConfig.childProjects;
        if (_childProjects && Array.isArray(_childProjects)) {
            _filePathStartIndex = 3;
            // 如果url中项目名后紧跟的是子项目名
            if (_childProjects.indexOf(paths[2]) >= 0) {
                dataPath = "data/" + projectName + "/" + paths[2] + "/" + dataName + ".json";
                configPath = "data/" + projectName + "/" + paths[2] + "/config/" + pageName + ".json";
                childProjectName = "/" + paths[2];
            }
        }
        else {
            dataPath = "data/" + projectName + "/" + dataName + ".json";
            configPath = "data/" + projectName + "/config/" + pageName + ".json";
        }
        for (var i = _filePathStartIndex; i < len; i++) {
            if (i < len - 1 && i > 0) {
                filePath += '/' + paths[i];
            }
        }
        var objData = {}, configData = {}, initModel = {};
        dataPath = path.resolve(rootDir, dataPath);
        configPath = path.resolve(rootDir, configPath);
        if (fs.existsSync(dataPath)) {
            try {
                var _data = fs.readFileSync(path.resolve(rootDir, dataPath), 'utf8');
                objData = Object.assign(JSON.parse(_data.toString()));
            }
            catch (error) {
                log_1["default"].error(error);
            }
        }
        else {
            log_1["default"].error(dataPath + " \u4E0D\u5B58\u5728");
        }
        if (fs.existsSync(configPath)) {
            try {
                var _configBit = fs.readFileSync(path.resolve(rootDir, configPath), 'utf8');
                configData = Object.assign(configData, JSON.parse(_configBit.toString()));
            }
            catch (error) {
                log_1["default"].error(error);
            }
        }
        else {
            log_1["default"].warn(configPath + " \u4E0D\u5B58\u5728");
        }
        initModel = Object.assign(objData, configData);
        // 如果子项目 config.js 有配置initModelHandle, 则运行对initModel进行处理
        projectConfig.initModelHandle && projectConfig.initModelHandle(initModel);
        // 将项目名传给vm引擎, 传递给res.render的第二个参数....取一个奇怪的名字(__VMOPTIONS__)防止冲突
        Object.assign(initModel, {
            __VMOPTIONS__: {
                childProjectName: childProjectName,
                projectConfig: projectConfig
            }
        });
        log_1["default"].debug("\u6E32\u67D3vm\u6587\u4EF6: " + projectName + "/build/WEB-INF" + childProjectName + filePath + ".vm");
        res.render(projectName + '/build/WEB-INF' + childProjectName + filePath + '.vm', initModel);
    });
    return router;
};
exports["default"] = initRoutes;
