"use strict";
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var log_1 = require("../../util/log");
var express = require("express");
var cookieParser = require("cookie-parser");
var bodyParser = require("body-parser");
var velocity_mjd_1 = require("velocity-mjd");
var proxyMid = require("http-proxy-middleware");
var initRoutes_1 = require("./initRoutes");
// rewrite中间件
var rewrite_1 = require("./middleware/rewrite");
// mock中间件
var mock_1 = require("./middleware/mock");
// 自动补全静态文件子项目名中间件
var mapStatic_1 = require("./middleware/mapStatic");
var initServer = function (rootConfig, rootPath) {
    var doProxy = rootConfig && rootConfig.proxy, doRewrite = rootConfig && rootConfig.rewrite;
    var app = express();
    var paths = [];
    var rootDir = rootPath + '';
    // view engine setup
    app.engine('vm', function (filePath, options, callback) {
        /**
         * @param {string} filePath - vm路径
         * @param {object} options - vm数据object
         * @param {object} options.__VMOPTIONS__ - 路由传递过来的参数
         * @param {function | undefined} callback - vm回调
         */
        var targetVM = "";
        var projectName = "";
        log_1["default"].success("vm\u5F00\u59CB\u6E32\u67D3: " + path.relative(rootDir, filePath));
        if (filePath.indexOf('.vm') !== -1) {
            var content = fs.readFileSync(filePath, 'utf8');
            if (content.indexOf('$layout') !== -1) {
                var reg = /\$layout(\s*)=(\s*)"(.*\.vm)"/;
                var matchs = content.match(reg);
                if (matchs && matchs.length > 0) {
                    var files = filePath.split('build');
                    var _projectSplitArr = filePath.split('project');
                    targetVM = _projectSplitArr[_projectSplitArr.length - 1];
                    projectName = targetVM.split(path.sep)[1] || targetVM.split('\\')[1];
                    // 如果存在子项目, 需要调整layout目录路径
                    if (options.__VMOPTIONS__ && options.__VMOPTIONS__.childProjectName) {
                        var childProjectName = options.__VMOPTIONS__.childProjectName;
                        filePath = files[0] + "build/WEB-INF/" + childProjectName + "/layout/" + matchs[3];
                    }
                    else {
                        filePath = files[0] + 'build/WEB-INF/layout/' + matchs[3];
                    }
                }
            }
        }
        log_1["default"].debug("vm root: " + path.resolve(rootDir, 'project'));
        log_1["default"].debug("vm layout: " + filePath);
        var engine = new velocity_mjd_1.Engine({
            encoding: 'utf-8',
            root: path.resolve(rootDir, 'project'),
            template: filePath,
            targetVM: "." + targetVM
        });
        options.Double = {
            parseDouble: function (str) {
                return parseFloat(str);
            },
            parseInt: function (str) {
                return parseInt(str);
            },
            parseString: function (str) {
                return str + "";
            }
        };
        options['utils'] = {
            tostring: function (obj) {
                return JSON.stringify(obj);
            },
            checkTimes: function (dateTime) {
                var patten = /[0-9]{1,2}:00-[0-9]{1,2}:00/g;
                return patten.test(dateTime);
            }
        };
        var lnPath = path.resolve(rootDir, 'data/ln/m_cn.json');
        var lnPath2 = path.resolve(rootDir, 'data/ln/m_cn_' + projectName + '.json');
        if (fs.existsSync(lnPath2) || fs.existsSync(lnPath)) {
            var _data = void 0;
            if (fs.existsSync(lnPath2)) {
                _data = fs.readFileSync(path.resolve(rootDir, lnPath2));
            }
            else {
                _data = fs.readFileSync(path.resolve(rootDir, lnPath));
            }
            var lnData_1 = Object.assign(JSON.parse(_data.toString()));
            log_1["default"].debug('语言包获取成功');
            options["res"] = {
                getText: function (textKey) {
                    return lnData_1[textKey];
                },
                insertText: function () {
                    var _arg = Array.prototype.slice.call(arguments);
                    var _key = _arg[0];
                    var _values = _arg.slice(1);
                    if (_values[0].indexOf('_') > -1) {
                        _values = _values[0].split('_');
                    }
                    var text = lnData_1[_key], res = text.replace(/\{(\d)\}/g, function (result, index) {
                        return _values[index] || '';
                    });
                    return res;
                }
            };
        }
        else {
            log_1["default"].warn("\u9879\u76EE\u8BED\u8A00\u5305 " + lnPath + " \u4E0D\u5B58\u5728");
        }
        options["murl"] = {
            /**
             * 清除字符串
             *
             * @param {String} str1 - 源字符串
             * @param {String} str2 - 清除掉字符串
             * @return {String} - 结果
             */
            splitStr: function (str1, str2) {
                if (str1 && str2) {
                    if (str1.indexOf(str2) !== -1) {
                        return str1.replace(str2, "");
                    }
                }
            },
            /**
             * 合并请求地址，将sid附加到地址里面ʽ
             * @param {String} base -
             * @param {String} sid -
             * @return {String} -
             */
            composite: function (base, sid) {
                var url;
                var paramStr = base.indexOf("?") !== -1 ? "&" : "?";
                if (base && sid) {
                    url = base + paramStr + "sid=" + escape(sid.replace(new RegExp("[\'\"\\s]", "gm"), ""));
                }
                else {
                    url = '';
                }
                return url.toString();
            }
        };
        options.NumberTool = {
            format: function (fort, str) {
                return str + '.00';
            }
        };
        var rendered = engine.render(options);
        log_1["default"].debug('vm渲染成功');
        return callback(null, rendered);
    });
    // view engine setup
    /*
     * 配置模板路径
     * */
    paths.push(path.join(rootDir, './project'));
    app.set('views', paths);
    app.set('view engine', 'vm');
    app.set('view cache', false);
    // 反向代理中间件
    doProxy && doProxy.forEach(function (proxy) {
        proxy.config.logLevel = 'warn';
        log_1["default"].debug("\u53CD\u4EE3: " + proxy.route + " => " + proxy.config.target);
        app.use(proxy.route, proxyMid(proxy.config));
    });
    // 本地静态文件rewrite
    app.use(rewrite_1["default"](rootConfig, rootDir));
    app.use(bodyParser.json());
    app.use(bodyParser.urlencoded({
        extended: false
    }));
    app.use(cookieParser());
    // data目录mock配置
    app.use('/data', mock_1["default"](rootDir));
    // 所有子项目静态文件服务
    var projectDir = path.resolve(rootDir, 'project'), projectChildren = fs.readdirSync(projectDir);
    projectChildren.forEach(function (proj) {
        // 排除以'.'开头的unix系统隐藏文件夹
        if (proj.indexOf('.') !== 0) {
            var projectConfigPath = path.resolve(projectDir, proj, 'config.js');
            if (fs.existsSync(projectConfigPath)) {
                log_1["default"].debug("\u9759\u6001\u8D44\u6E90\u6258\u7BA1 " + proj);
                // 对于根目录有config.js的目录做文件托管
                app.use("/" + proj, express.static(path.join(rootDir, "project/" + proj + "/build")));
                app.use("/static/" + proj, express.static(path.join(rootDir, "project/" + proj + "/build")));
                // 如果有子目录
                if (require(projectConfigPath).childProjects) {
                    var _childProjects = require(projectConfigPath).childProjects;
                    for (var _childP in _childProjects) {
                        app.use("/" + proj + "/" + _childP, express.static(path.join(rootDir, "project/" + proj + "/build")));
                    }
                }
            }
        }
    });
    // 托管project/source/js-lab js文件
    app.use("/common/js", express.static(path.join(rootDir, "project/common/source/js-lab")));
    app.use("/common/lib", express.static(path.join(rootDir, "project/common/lib")));
    app.use(initRoutes_1["default"](rootPath));
    // 尝试补全子项目名
    app.use(mapStatic_1["default"](global.__PROJECT__, rootPath));
    app.use(function (req, res, next) {
        var err = new Error(' O(∩_∩)O~ dophin找不到这个文件或者无法mock这个请求~~');
        err.status = 404;
        next(err);
    });
    app.use(function (err, req, res, next) {
        res.status(err.status || 500);
        res.writeHead(200, {
            'Content-Language': 'zh-CN',
            'Content-Type': 'text/html;charset=UTF-8',
            'Cache-Control': 'max-age=0',
            'Server': 'dophin',
            'X-Powered-By': 'dophin'
        });
        res.end("<h1>\u9519\u8BEF\u7801\uFF1A" + (err.status || 500) + "</h1>\n        <h2 style=\"color:red\">" + err.message + "</h2>\n        <h2>\u8BF7\u6C42\u65B9\u5F0F\uFF1A" + req.method + "</h2>\n        <h2>\u8BF7\u6C42\u8DEF\u5F84\uFF1A" + req.url + "</h2>\n        <h3>\u9519\u8BEFstack\uFF1A</h3>\n        <div>" + err.stack + "</div>");
    });
    return app;
};
exports["default"] = initServer;
