"use strict";
exports.__esModule = true;
var fs = require("fs");
var path = require("path");
var dns = require("dns");
var http = require("http");
var log_1 = require("../../../util/log");
var dophin_core_1 = require("dophin-core");
var doRequest = function (request, response, host, port) {
    var options = {
        headers: request.headers,
        hostname: host,
        port: port || '80',
        method: request.method,
        path: request.url
    };
    var req = http.request(options, function (res) {
        response.writeHead(res.statusCode, res.headers);
        res.pipe(response);
    });
    req.on('error', function (err) {
        response.writeHeader(500, {
            'Content-Type': 'text/plain'
        });
        response.write('Request URL: ' + request.url);
        response.end('\nBreak Host Error: ' + err.message);
    });
    request.pipe(req);
};
// 转发到线上
var goOnline = function (request, response) {
    var hp = (request.headers.host || '').split(':');
    var host = hp[0];
    dns.resolve4(host, function (err, addresses) {
        if (err) {
            response.writeHeader(404, {
                'Content-Type': 'text/plain'
            });
            response.write('Request URL: ' + request.url);
            response.end('\nBreak Host Error: ' + err.message);
        }
        else {
            doRequest(request, response, addresses[0], hp[1]);
            log_1["default"].debug("\u8F6C\u53D1\u7EBF\u4E0A\u5730\u5740: " + addresses[0] + ":" + (hp[1] || 80) + request.url);
        }
    });
};
var initRewrite = function (rootConfig, rootPath) {
    var rootDir = rootPath + '';
    // 本地映射设置
    var defaultRewriteOption = {
        imageHandle: function (req, res, next) {
            var imgHandleReg = /\/(\w+)\/(\w+)\/([^\?]+)(\?.+)?$/;
            if (imgHandleReg.test(req.originalUrl)) {
                var _match = imgHandleReg.exec(req.originalUrl) || [];
                var _proj = _match[1], _type = _match[2], _filename = _match[3];
                // 先在本地找这个图片
                var projects = dophin_core_1.getAllProjects(rootPath + '');
                if (projects.includes(_proj)) {
                    if (_type === 'images') {
                        var imageLocalPath = path.resolve(rootDir, "project/" + _proj + "/build/" + _type, _filename);
                        if (fs.existsSync(imageLocalPath)) {
                            log_1["default"].info(_proj + "/" + _type + "/" + _filename + " \u6620\u5C04\u5230\u672C\u5730build\u6587\u4EF6\u5939");
                            res.writeHead(200, {
                                'Access-Control-Allow-Origin': '*',
                                'Content-Type': "image/" + path.extname(imageLocalPath).replace('.', ''),
                                'Cache-Control': 'max-age=0',
                                'Server': 'dophin',
                                'X-Powered-By': 'dophin'
                            });
                            res.end(fs.readFileSync(imageLocalPath));
                            return;
                        }
                    }
                }
            }
            log_1["default"].warn("\u672C\u5730\u4E0D\u5B58\u5728\u56FE\u7247" + req.originalUrl + ", \u8F6C\u53D1\u5230\u7EBF\u4E0A");
            goOnline(req, res);
        },
        handle: function (match, req, res, transform, next) {
            var _proj = match[1], _type = match[2], _filename = match[4];
            var _path = match[3], realFileName = _filename;
            _path = _path ? _path : ''; // _path很可能是undefined
            // 变形金刚系统   xxx => yyy
            if (Array.isArray(transform) && transform.length > 0) {
                for (var _i = 0, transform_1 = transform; _i < transform_1.length; _i++) {
                    var rule = transform_1[_i];
                    if ((typeof rule[0] === 'string' && _filename.indexOf(rule[0]) > -1) ||
                        (typeof rule[0] === 'object' && rule[0].test(_filename))) {
                        realFileName = _filename.replace(rule[0], rule[1]);
                        break;
                    }
                }
            }
            var filepath = path.resolve(rootDir, "project/" + _proj + "/build/" + _type + _path + realFileName);
            if (!fs.existsSync(filepath)) {
                filepath = path.resolve(rootDir, "project/" + _proj + "/build/" + _type + _path + _filename);
            }
            if (fs.existsSync(filepath)) {
                log_1["default"].info("" + _type + _path + _filename + " \u6620\u5C04\u5230\u672C\u5730build\u6587\u4EF6\u5939");
                var _content = fs.readFileSync(filepath);
                _content = "/**\n * \u6307\u5411\u672C\u5730\u6587\u4EF6\n * " + filepath + "\n */\n\n\n" + _content;
                if (_type === 'js') {
                    res.writeHead(200, {
                        'Access-Control-Allow-Origin': '*',
                        'Content-Type': 'application/x-javascript',
                        'Cache-Control': 'max-age=0',
                        'Server': 'dophin',
                        'X-Powered-By': 'dophin'
                    });
                }
                else {
                    res.writeHead(200, {
                        'Access-Control-Allow-Origin': '*',
                        'Content-Type': 'text/css',
                        'Cache-Control': 'max-age=0',
                        'Server': 'dophin',
                        'X-Powered-By': 'dophin'
                    });
                }
                res.end(_content);
            }
            else {
                // 走下一个路由
                // log.info(`本地${_proj}项目不存在${_type}${_path}${_filename}, 继续走下一个路由`);
                // next();
                // 或者转发到线上
                log_1["default"].warn("\u672C\u5730" + _proj + "\u9879\u76EE\u4E0D\u5B58\u5728" + _type + _path + _filename + ", \u8F6C\u53D1\u5230\u7EBF\u4E0A");
                goOnline(req, res);
            }
        }
    };
    var rewrite = function (req, res, next) {
        var reWriteConfig = rootConfig && rootConfig.rewrite;
        var _rewrite;
        // 是否设置该域名
        for (var _i = 0, _a = reWriteConfig; _i < _a.length; _i++) {
            var site = _a[_i];
            if (Array.isArray(site.host) && (site.host.indexOf(req.hostname) > -1)) {
                _rewrite = site;
                break;
            }
            if (req.hostname === site.host) {
                _rewrite = site;
                break;
            }
        }
        // 先判断是不是图片, 图片走 imageHandle
        var imgReg = /^.+\.(png|jpg|gif|apng|webp|jpeg|bmp|'还有谁')/i;
        if (_rewrite && imgReg.test(req.originalUrl)) {
            defaultRewriteOption.imageHandle(req, res, next);
            return;
        }
        // js|css进入 handle
        if (_rewrite && _rewrite.reg.test(req.originalUrl)) {
            var match = _rewrite.reg.exec(req.originalUrl);
            defaultRewriteOption.handle(match, req, res, _rewrite.transform, next);
            return;
        }
        else {
            next();
        }
    };
    return rewrite;
};
exports["default"] = initRewrite;
